# -*- coding: utf-8 -*-
'''
    :codeauthor: Mike Place <mp@saltstack.com>
'''

# Import python libs
from __future__ import absolute_import
import copy
import os

# Import Salt Testing libs
from tests.support.unit import TestCase, skipIf
from tests.support.mock import NO_MOCK, NO_MOCK_REASON, patch, MagicMock
from tests.support.mixins import AdaptedConfigurationTestCaseMixin
from tests.support.helpers import skip_if_not_root
# Import salt libs
import salt.minion
import salt.utils.event as event
from salt.exceptions import SaltSystemExit, SaltMasterUnresolvableError
import salt.syspaths
import tornado
import tornado.testing
from salt.ext.six.moves import range


__opts__ = {}


@skipIf(NO_MOCK, NO_MOCK_REASON)
class MinionTestCase(TestCase, AdaptedConfigurationTestCaseMixin):
    def test_invalid_master_address(self):
        with patch.dict(__opts__, {'ipv6': False, 'master': float('127.0'), 'master_port': '4555', 'retry_dns': False}):
            self.assertRaises(SaltSystemExit, salt.minion.resolve_dns, __opts__)

    def test_source_int_name_local(self):
        '''
        test when file_client local and
        source_interface_name is set
        '''
        interfaces = {'bond0.1234': {'hwaddr': '01:01:01:d0:d0:d0',
                                     'up': True, 'inet':
                                     [{'broadcast': '111.1.111.255',
                                     'netmask': '111.1.0.0',
                                     'label': 'bond0',
                                     'address': '111.1.0.1'}]}}
        with patch.dict(__opts__, {'ipv6': False, 'master': '127.0.0.1',
                                   'master_port': '4555', 'file_client': 'local',
                                   'source_interface_name': 'bond0.1234',
                                   'source_ret_port': 49017,
                                   'source_publish_port': 49018}), \
            patch('salt.utils.network.interfaces',
                  MagicMock(return_value=interfaces)):
            assert salt.minion.resolve_dns(__opts__) == {'master_ip': '127.0.0.1',
                                                         'source_ip': '111.1.0.1',
                                                         'source_ret_port': 49017,
                                                         'source_publish_port': 49018,
                                                         'master_uri': 'tcp://127.0.0.1:4555'}

    def test_source_int_name_remote(self):
        '''
        test when file_client remote and
        source_interface_name is set and
        interface is down
        '''
        interfaces = {'bond0.1234': {'hwaddr': '01:01:01:d0:d0:d0',
                                     'up': False, 'inet':
                                     [{'broadcast': '111.1.111.255',
                                     'netmask': '111.1.0.0',
                                     'label': 'bond0',
                                     'address': '111.1.0.1'}]}}
        with patch.dict(__opts__, {'ipv6': False, 'master': '127.0.0.1',
                                   'master_port': '4555', 'file_client': 'remote',
                                   'source_interface_name': 'bond0.1234',
                                   'source_ret_port': 49017,
                                   'source_publish_port': 49018}), \
            patch('salt.utils.network.interfaces',
                  MagicMock(return_value=interfaces)):
            assert salt.minion.resolve_dns(__opts__) == {'master_ip': '127.0.0.1',
                                                         'source_ret_port': 49017,
                                                         'source_publish_port': 49018,
                                                         'master_uri': 'tcp://127.0.0.1:4555'}

    def test_source_address(self):
        '''
        test when source_address is set
        '''
        interfaces = {'bond0.1234': {'hwaddr': '01:01:01:d0:d0:d0',
                                     'up': False, 'inet':
                                     [{'broadcast': '111.1.111.255',
                                     'netmask': '111.1.0.0',
                                     'label': 'bond0',
                                     'address': '111.1.0.1'}]}}
        with patch.dict(__opts__, {'ipv6': False, 'master': '127.0.0.1',
                                   'master_port': '4555', 'file_client': 'local',
                                   'source_interface_name': '',
                                   'source_address': '111.1.0.1',
                                   'source_ret_port': 49017,
                                   'source_publish_port': 49018}), \
            patch('salt.utils.network.interfaces',
                  MagicMock(return_value=interfaces)):
            assert salt.minion.resolve_dns(__opts__) == {'source_publish_port': 49018,
                                                         'source_ret_port': 49017,
                                                         'master_uri': 'tcp://127.0.0.1:4555',
                                                         'source_ip': '111.1.0.1',
                                                         'master_ip': '127.0.0.1'}

    # Tests for _handle_decoded_payload in the salt.minion.Minion() class: 3

    def test_handle_decoded_payload_jid_match_in_jid_queue(self):
        '''
        Tests that the _handle_decoded_payload function returns when a jid is given that is already present
        in the jid_queue.

        Note: This test doesn't contain all of the patch decorators above the function like the other tests
        for _handle_decoded_payload below. This is essential to this test as the call to the function must
        return None BEFORE any of the processes are spun up because we should be avoiding firing duplicate
        jobs.
        '''
        mock_opts = salt.config.DEFAULT_MINION_OPTS
        mock_data = {'fun': 'foo.bar',
                     'jid': 123}
        mock_jid_queue = [123]
        minion = salt.minion.Minion(mock_opts, jid_queue=copy.copy(mock_jid_queue), io_loop=tornado.ioloop.IOLoop())
        try:
            ret = minion._handle_decoded_payload(mock_data).result()
            self.assertEqual(minion.jid_queue, mock_jid_queue)
            self.assertIsNone(ret)
        finally:
            minion.destroy()

    def test_handle_decoded_payload_jid_queue_addition(self):
        '''
        Tests that the _handle_decoded_payload function adds a jid to the minion's jid_queue when the new
        jid isn't already present in the jid_queue.
        '''
        with patch('salt.minion.Minion.ctx', MagicMock(return_value={})), \
                patch('salt.utils.process.SignalHandlingMultiprocessingProcess.start', MagicMock(return_value=True)), \
                patch('salt.utils.process.SignalHandlingMultiprocessingProcess.join', MagicMock(return_value=True)):
            mock_jid = 11111
            mock_opts = salt.config.DEFAULT_MINION_OPTS
            mock_data = {'fun': 'foo.bar',
                         'jid': mock_jid}
            mock_jid_queue = [123, 456]
            minion = salt.minion.Minion(mock_opts, jid_queue=copy.copy(mock_jid_queue), io_loop=tornado.ioloop.IOLoop())
            try:

                # Assert that the minion's jid_queue attribute matches the mock_jid_queue as a baseline
                # This can help debug any test failures if the _handle_decoded_payload call fails.
                self.assertEqual(minion.jid_queue, mock_jid_queue)

                # Call the _handle_decoded_payload function and update the mock_jid_queue to include the new
                # mock_jid. The mock_jid should have been added to the jid_queue since the mock_jid wasn't
                # previously included. The minion's jid_queue attribute and the mock_jid_queue should be equal.
                minion._handle_decoded_payload(mock_data).result()
                mock_jid_queue.append(mock_jid)
                self.assertEqual(minion.jid_queue, mock_jid_queue)
            finally:
                minion.destroy()

    def test_handle_decoded_payload_jid_queue_reduced_minion_jid_queue_hwm(self):
        '''
        Tests that the _handle_decoded_payload function removes a jid from the minion's jid_queue when the
        minion's jid_queue high water mark (minion_jid_queue_hwm) is hit.
        '''
        with patch('salt.minion.Minion.ctx', MagicMock(return_value={})), \
                patch('salt.utils.process.SignalHandlingMultiprocessingProcess.start', MagicMock(return_value=True)), \
                patch('salt.utils.process.SignalHandlingMultiprocessingProcess.join', MagicMock(return_value=True)):
            mock_opts = salt.config.DEFAULT_MINION_OPTS
            mock_opts['minion_jid_queue_hwm'] = 2
            mock_data = {'fun': 'foo.bar',
                         'jid': 789}
            mock_jid_queue = [123, 456]
            minion = salt.minion.Minion(mock_opts, jid_queue=copy.copy(mock_jid_queue), io_loop=tornado.ioloop.IOLoop())
            try:

                # Assert that the minion's jid_queue attribute matches the mock_jid_queue as a baseline
                # This can help debug any test failures if the _handle_decoded_payload call fails.
                self.assertEqual(minion.jid_queue, mock_jid_queue)

                # Call the _handle_decoded_payload function and check that the queue is smaller by one item
                # and contains the new jid
                minion._handle_decoded_payload(mock_data).result()
                self.assertEqual(len(minion.jid_queue), 2)
                self.assertEqual(minion.jid_queue, [456, 789])
            finally:
                minion.destroy()

    def test_process_count_max(self):
        '''
        Tests that the _handle_decoded_payload function does not spawn more than the configured amount of processes,
        as per process_count_max.
        '''
        with patch('salt.minion.Minion.ctx', MagicMock(return_value={})), \
                patch('salt.utils.process.SignalHandlingMultiprocessingProcess.start', MagicMock(return_value=True)), \
                patch('salt.utils.process.SignalHandlingMultiprocessingProcess.join', MagicMock(return_value=True)), \
                patch('salt.utils.minion.running', MagicMock(return_value=[])), \
                patch('tornado.gen.sleep', MagicMock(return_value=tornado.concurrent.Future())):
            process_count_max = 10
            mock_opts = salt.config.DEFAULT_MINION_OPTS
            mock_opts['minion_jid_queue_hwm'] = 100
            mock_opts["process_count_max"] = process_count_max

            io_loop = tornado.ioloop.IOLoop()
            minion = salt.minion.Minion(mock_opts, jid_queue=[], io_loop=io_loop)
            try:

                # mock gen.sleep to throw a special Exception when called, so that we detect it
                class SleepCalledEception(Exception):
                    """Thrown when sleep is called"""
                    pass
                tornado.gen.sleep.return_value.set_exception(SleepCalledEception())

                # up until process_count_max: gen.sleep does not get called, processes are started normally
                for i in range(process_count_max):
                    mock_data = {'fun': 'foo.bar',
                                 'jid': i}
                    io_loop.run_sync(lambda data=mock_data: minion._handle_decoded_payload(data))
                    self.assertEqual(salt.utils.process.SignalHandlingMultiprocessingProcess.start.call_count, i + 1)
                    self.assertEqual(len(minion.jid_queue), i + 1)
                    salt.utils.minion.running.return_value += [i]

                # above process_count_max: gen.sleep does get called, JIDs are created but no new processes are started
                mock_data = {'fun': 'foo.bar',
                             'jid': process_count_max + 1}

                self.assertRaises(SleepCalledEception,
                                  lambda: io_loop.run_sync(lambda: minion._handle_decoded_payload(mock_data)))
                self.assertEqual(salt.utils.process.SignalHandlingMultiprocessingProcess.start.call_count,
                                 process_count_max)
                self.assertEqual(len(minion.jid_queue), process_count_max + 1)
            finally:
                minion.destroy()

    def test_beacons_before_connect(self):
        '''
        Tests that the 'beacons_before_connect' option causes the beacons to be initialized before connect.
        '''
        with patch('salt.minion.Minion.ctx', MagicMock(return_value={})), \
                patch('salt.minion.Minion.sync_connect_master', MagicMock(side_effect=RuntimeError('stop execution'))), \
                patch('salt.utils.process.SignalHandlingMultiprocessingProcess.start', MagicMock(return_value=True)), \
                patch('salt.utils.process.SignalHandlingMultiprocessingProcess.join', MagicMock(return_value=True)):
            mock_opts = self.get_config('minion', from_scratch=True)
            mock_opts['beacons_before_connect'] = True
            io_loop = tornado.ioloop.IOLoop()
            io_loop.make_current()
            minion = salt.minion.Minion(mock_opts, io_loop=io_loop)
            try:

                try:
                    minion.tune_in(start=True)
                except RuntimeError:
                    pass

                # Make sure beacons are initialized but the sheduler is not
                self.assertTrue('beacons' in minion.periodic_callbacks)
                self.assertTrue('schedule' not in minion.periodic_callbacks)
            finally:
                minion.destroy()

    def test_scheduler_before_connect(self):
        '''
        Tests that the 'scheduler_before_connect' option causes the scheduler to be initialized before connect.
        '''
        with patch('salt.minion.Minion.ctx', MagicMock(return_value={})), \
                patch('salt.minion.Minion.sync_connect_master', MagicMock(side_effect=RuntimeError('stop execution'))), \
                patch('salt.utils.process.SignalHandlingMultiprocessingProcess.start', MagicMock(return_value=True)), \
                patch('salt.utils.process.SignalHandlingMultiprocessingProcess.join', MagicMock(return_value=True)):
            mock_opts = self.get_config('minion', from_scratch=True)
            mock_opts['scheduler_before_connect'] = True
            io_loop = tornado.ioloop.IOLoop()
            io_loop.make_current()
            minion = salt.minion.Minion(mock_opts, io_loop=io_loop)
            try:
                try:
                    minion.tune_in(start=True)
                except RuntimeError:
                    pass

                # Make sure the scheduler is initialized but the beacons are not
                self.assertTrue('schedule' in minion.periodic_callbacks)
                self.assertTrue('beacons' not in minion.periodic_callbacks)
            finally:
                minion.destroy()

    def test_minion_retry_dns_count(self):
        '''
        Tests that the resolve_dns will retry dns look ups for a maximum of
        3 times before raising a SaltMasterUnresolvableError exception.
        '''
        with patch.dict(__opts__, {'ipv6': False, 'master': 'dummy',
                                   'master_port': '4555',
                                   'retry_dns': 1, 'retry_dns_count': 3}):
            self.assertRaises(SaltMasterUnresolvableError,
                              salt.minion.resolve_dns, __opts__)


@skipIf(NO_MOCK, NO_MOCK_REASON)
class MinionAsyncTestCase(TestCase, AdaptedConfigurationTestCaseMixin, tornado.testing.AsyncTestCase):

    @skip_if_not_root
    def test_sock_path_len(self):
        '''
        This tests whether or not a larger hash causes the sock path to exceed
        the system's max sock path length. See the below link for more
        information.

        https://github.com/saltstack/salt/issues/12172#issuecomment-43903643
        '''
        opts = {
            'id': 'salt-testing',
            'hash_type': 'sha512',
            'sock_dir': os.path.join(salt.syspaths.SOCK_DIR, 'minion'),
            'extension_modules': ''
        }
        with patch.dict(__opts__, opts):
            try:
                event_publisher = event.AsyncEventPublisher(__opts__)
                result = True
            except ValueError:
                #  There are rare cases where we operate a closed socket, especially in containers.
                # In this case, don't fail the test because we'll catch it down the road.
                result = True
            except SaltSystemExit:
                result = False
        self.assertTrue(result)



# -*- coding: utf-8 -*-
'''
    :codeauthor: Nicole Thomas (nicole@saltstack.com)
'''

# Import Python Libs
from __future__ import absolute_import, print_function, unicode_literals
from inspect import ArgSpec
import logging

# Import Salt Libs
import salt.states.module as module

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import skipIf, TestCase
from tests.support.mock import (
    NO_MOCK,
    NO_MOCK_REASON,
    MagicMock,
    patch
)

log = logging.getLogger(__name__)

CMD = 'foo.bar'

STATE_APPLY_RET = {
    'module_|-test2_|-state.apply_|-run': {
        'comment': 'Module function state.apply executed',
        'name': 'state.apply',
        'start_time': '16:11:48.818932',
        'result': False,
        'duration': 179.439,
        '__run_num__': 0,
        'changes': {
            'ret': {
                'module_|-test3_|-state.apply_|-run': {
                    'comment': 'Module function state.apply executed',
                    'name': 'state.apply',
                    'start_time': '16:11:48.904796',
                    'result': True,
                    'duration': 89.522,
                    '__run_num__': 0,
                    'changes': {
                        'ret': {
                            'module_|-test4_|-cmd.run_|-run': {
                                'comment': 'Module function cmd.run executed',
                                'name': 'cmd.run',
                                'start_time': '16:11:48.988574',
                                'result': True,
                                'duration': 4.543,
                                '__run_num__': 0,
                                'changes': {
                                    'ret': 'Wed Mar  7 16:11:48 CET 2018'
                                },
                                '__id__': 'test4'
                            }
                        }
                    },
                    '__id__': 'test3'
                },
                'module_|-test3_fail_|-test3_fail_|-run': {
                    'comment': 'Module function test3_fail is not available',
                    'name': 'test3_fail',
                    'start_time': '16:11:48.994607',
                    'result': False,
                    'duration': 0.466,
                    '__run_num__': 1,
                    'changes': {},
                    '__id__': 'test3_fail'
                }
            }
        },
        '__id__': 'test2'
    }
}


def _mocked_func_named(name, names=('Fred', 'Swen',)):
    '''
    Mocked function with named defaults.

    :param name:
    :param names:
    :return:
    '''
    return {'name': name, 'names': names}


def _mocked_func_args(*args):
    '''
    Mocked function with args.

    :param args:
    :return:
    '''
    assert args == ('foo', 'bar')
    return {'args': args}


def _mocked_none_return(ret=None):
    '''
    Mocked function returns None
    :return:
    '''
    return ret


@skipIf(NO_MOCK, NO_MOCK_REASON)
class ModuleStateTest(TestCase, LoaderModuleMockMixin):
    '''
    Tests module state (salt/states/module.py)
    '''
    def setup_loader_modules(self):
        return {
            module: {
                '__opts__': {'test': False},
                '__salt__': {CMD: MagicMock()}
            }
        }

    @classmethod
    def setUpClass(cls):
        cls.aspec = ArgSpec(args=['hello', 'world'],
                            varargs=None,
                            keywords=None,
                            defaults=False)

        cls.bspec = ArgSpec(args=[],
                            varargs='names',
                            keywords='kwargs',
                            defaults=None)

    @classmethod
    def tearDownClass(cls):
        del cls.aspec
        del cls.bspec

    def test_run_module_not_available(self):
        '''
        Tests the return of module.run state when the module function is not available.
        :return:
        '''
        with patch.dict(module.__salt__, {}, clear=True):
            with patch.dict(module.__opts__, {'use_superseded': ['module.run']}):
                ret = module.run(**{CMD: None})
                if ret['comment'] != "Unavailable function: {0}.".format(CMD) or ret['result']:
                    self.fail('module.run did not fail as expected: {0}'.format(ret))

    def test_module_run_hidden_varargs(self):
        '''
        Tests the return of module.run state when hidden varargs are used with
        wrong type.
        '''
        with patch('salt.utils.args.get_function_argspec', MagicMock(return_value=self.bspec)):
            ret = module._run(CMD, m_names='anyname')
            comment = "'names' must be a list."
            self.assertEqual(ret['comment'], comment)

    def test_run_testmode(self):
        '''
        Tests the return of the module.run state when test=True is passed.
        :return:
        '''
        with patch.dict(module.__opts__, {'test': True, 'use_superseded': ['module.run']}):
            ret = module.run(**{CMD: None})
            if ret['comment'] != "Function {0} to be executed.".format(CMD) or not ret['result']:
                self.fail('module.run failed: {0}'.format(ret))

    def test_run_missing_arg(self):
        '''
        Tests the return of module.run state when arguments are missing
        :return:
        '''
        with patch.dict(module.__salt__, {CMD: _mocked_func_named}):
            with patch.dict(module.__opts__, {'use_superseded': ['module.run']}):
                ret = module.run(**{CMD: None})
                expected_comment = "'{0}' failed: Function expects 1 parameters, got only 0".format(CMD)
                if ret['comment'] != expected_comment:
                    self.fail('module.run did not fail as expected: {0}'.format(ret))

    def test_run_correct_arg(self):
        '''
        Tests the return of module.run state when arguments are correct
        :return:
        '''
        with patch.dict(module.__salt__, {CMD: _mocked_func_named}):
            with patch.dict(module.__opts__, {'use_superseded': ['module.run']}):
                ret = module.run(**{CMD: ['Fred']})
                if ret['comment'] != '{0}: Success'.format(CMD) or not ret['result']:
                    self.fail('module.run failed: {0}'.format(ret))

    def test_run_state_apply_result_false(self):
        '''
        Tests the 'result' of module.run that calls state.apply execution module
        :return:
        '''
        with patch.dict(module.__salt__, {"state.apply": MagicMock(return_value=STATE_APPLY_RET)}):
            with patch.dict(module.__opts__, {'use_deprecated': ['module.run']}):
                ret = module.run(**{"name": "state.apply", 'mods': 'test2'})
                if ret['result']:
                    self.fail('module.run did not report false result: {0}'.format(ret))

    def test_run_unexpected_keywords(self):
        with patch.dict(module.__salt__, {CMD: _mocked_func_args}):
            with patch.dict(module.__opts__, {'use_superseded': ['module.run']}):
                ret = module.run(**{CMD: [{'foo': 'bar'}]})
                expected_comment = "'{0}' failed: {1}() got an unexpected keyword argument " \
                                   "'foo'".format(CMD, module.__salt__[CMD].__name__)
                if ret['comment'] != expected_comment or ret['result']:
                    self.fail('module.run did not fail as expected: {0}'.format(ret))

    def test_run_args(self):
        '''
        Test unnamed args.
        :return:
        '''
        with patch.dict(module.__salt__, {CMD: _mocked_func_args}):
            with patch.dict(module.__opts__, {'use_superseded': ['module.run']}):
                try:
                    ret = module.run(**{CMD: ['foo', 'bar']})
                except Exception as exc:
                    log.exception('test_run_none_return: raised exception')
                    self.fail('module.run raised exception: {0}'.format(exc))
                if not ret['result']:
                    log.exception(
                        'test_run_none_return: test failed, result: %s',
                        ret
                    )
                    self.fail('module.run failed: {0}'.format(ret))

    def test_run_none_return(self):
        '''
        Test handling of a broken function that returns None.
        :return:
        '''
        with patch.dict(module.__salt__, {CMD: _mocked_none_return}):
            with patch.dict(module.__opts__, {'use_superseded': ['module.run']}):
                try:
                    ret = module.run(**{CMD: None})
                except Exception as exc:
                    log.exception('test_run_none_return: raised exception')
                    self.fail('module.run raised exception: {0}'.format(exc))
                if not ret['result']:
                    log.exception(
                        'test_run_none_return: test failed, result: %s',
                        ret
                    )
                    self.fail('module.run failed: {0}'.format(ret))

    def test_run_typed_return(self):
        '''
        Test handling of a broken function that returns any type.
        :return:
        '''
        for val in [1, 0, 'a', '', (1, 2,), (), [1, 2], [], {'a': 'b'}, {}, True, False]:
            with patch.dict(module.__salt__, {CMD: _mocked_none_return}):
                with patch.dict(module.__opts__, {'use_superseded': ['module.run']}):
                    log.debug('test_run_typed_return: trying %s', val)
                    try:
                        ret = module.run(**{CMD: [{'ret': val}]})
                    except Exception as exc:
                        log.exception('test_run_typed_return: raised exception')
                        self.fail('module.run raised exception: {0}'.format(exc))
                    if not ret['result']:
                        log.exception(
                            'test_run_typed_return: test failed, result: %s',
                            ret
                        )
                        self.fail('module.run failed: {0}'.format(ret))

    def test_run_batch_call(self):
        '''
        Test batch call
        :return:
        '''
        with patch.dict(module.__opts__, {'use_superseded': ['module.run']}):
            with patch.dict(module.__salt__,
                            {'first': _mocked_none_return,
                             'second': _mocked_none_return,
                             'third': _mocked_none_return}, clear=True):
                for f_name in module.__salt__:
                    log.debug('test_run_batch_call: trying %s', f_name)
                    try:
                        ret = module.run(**{f_name: None})
                    except Exception as exc:
                        log.exception('test_run_batch_call: raised exception')
                        self.fail('module.run raised exception: {0}'.format(exc))
                    if not ret['result']:
                        log.exception(
                            'test_run_batch_call: test failed, result: %s',
                            ret
                        )
                        self.fail('module.run failed: {0}'.format(ret))

    def test_module_run_module_not_available(self):
        '''
        Tests the return of module.run state when the module function
        name isn't available
        '''
        with patch.dict(module.__salt__, {}, clear=True):
            ret = module._run(CMD)
            comment = 'Module function {0} is not available'.format(CMD)
            self.assertEqual(ret['comment'], comment)
            self.assertFalse(ret['result'])

    def test_module_run_test_true(self):
        '''
        Tests the return of module.run state when test=True is passed in
        '''
        with patch.dict(module.__opts__, {'test': True}):
            ret = module._run(CMD)
            comment = 'Module function {0} is set to execute'.format(CMD)
            self.assertEqual(ret['comment'], comment)

    def test_module_run_missing_arg(self):
        '''
        Tests the return of module.run state when arguments are missing
        '''
        with patch('salt.utils.args.get_function_argspec', MagicMock(return_value=self.aspec)):
            ret = module._run(CMD)
            comment = 'The following arguments are missing:'
            self.assertIn(comment, ret['comment'])
            self.assertIn('world', ret['comment'])
            self.assertIn('hello', ret['comment'])



# -*- coding: utf-8 -*-
'''
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
'''

# Import Python Libs
from __future__ import absolute_import, unicode_literals, print_function

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import TestCase, skipIf
from tests.support.mock import (
    NO_MOCK,
    NO_MOCK_REASON
)

# Import Salt Libs
import salt.modules.win_disk as win_disk


class MockKernel32(object):
    '''
    Mock windll class
    '''
    def __init__(self):
        pass

    @staticmethod
    def GetLogicalDrives():
        '''
        Mock GetLogicalDrives method
        '''
        return 1


class MockWindll(object):
    '''
    Mock windll class
    '''
    def __init__(self):
        self.kernel32 = MockKernel32()


class MockCtypes(object):
    '''
    Mock ctypes class
    '''
    def __init__(self):
        self.windll = MockWindll()


@skipIf(NO_MOCK, NO_MOCK_REASON)
class WinDiskTestCase(TestCase, LoaderModuleMockMixin):
    '''
    Test cases for salt.modules.win_disk
    '''
    def setup_loader_modules(self):
        return {win_disk: {'ctypes': MockCtypes()}}

    # 'usage' function tests: 1

    def test_usage(self):
        '''
        Test if it return usage information for volumes mounted on this minion.
        '''
        self.assertDictEqual(win_disk.usage(),
                             {'A:\\': {'available': None,
                                       '1K-blocks': None,
                                       'used': None,
                                       'capacity': None,
                                       'filesystem': 'A:\\'}})



# -*- coding: utf-8 -*-
'''
unit tests for the sqs_events engine
'''
# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import skipIf, TestCase
from tests.support.mock import (
    NO_MOCK,
    NO_MOCK_REASON,
    MagicMock,
    patch)

# Import Salt Libs
import salt.engines.sqs_events as sqs_events


@skipIf(sqs_events.HAS_BOTO is False, 'The boto library is not installed')
@skipIf(NO_MOCK, NO_MOCK_REASON)
class EngineSqsEventTestCase(TestCase, LoaderModuleMockMixin):
    '''
    Test cases for salt.engine.sqs_events
    '''

    def setup_loader_modules(self):
        patcher = patch('salt.engines.sqs_events.boto.sqs')
        self.mock_sqs = patcher.start()
        self.addCleanup(patcher.stop)
        self.addCleanup(delattr, self, 'mock_sqs')
        return {sqs_events: {}}

    def sample_msg(self):
        fake_msg = MagicMock()
        fake_msg.get_body.return_value = "This is a test message"
        fake_msg.delete.return_value = True
        return fake_msg

    # 'present' function tests: 1
    def test_no_queue_present(self):
        '''
        Test to ensure the SQS engine logs a warning when queue not present
        '''
        with patch('salt.engines.sqs_events.log') as mock_logging:
            with patch('time.sleep', return_value=None) as mock_sleep:
                q = None
                q_name = 'mysqs'
                mock_fire = MagicMock(return_value=True)
                sqs_events._process_queue(q, q_name, mock_fire)
                self.assertTrue(mock_logging.warning.called)
                self.assertFalse(self.mock_sqs.queue.Queue().get_messages.called)

    def test_minion_message_fires(self):
        '''
        Test SQS engine correctly gets and fires messages on minion
        '''
        msgs = [self.sample_msg(), self.sample_msg()]
        self.mock_sqs.queue.Queue().get_messages.return_value = msgs
        q = self.mock_sqs.queue.Queue()
        q_name = 'mysqs'
        mock_event = MagicMock(return_value=True)
        mock_fire = MagicMock(return_value=True)
        with patch.dict(sqs_events.__salt__, {'event.send': mock_event}):
            sqs_events._process_queue(q, q_name, mock_fire)
            self.assertTrue(self.mock_sqs.queue.Queue().get_messages.called)
            self.assertTrue(all(x.delete.called for x in msgs))

    def test_master_message_fires(self):
        '''
        Test SQS engine correctly gets and fires messages on master
        '''
        msgs = [self.sample_msg(), self.sample_msg()]
        self.mock_sqs.queue.Queue().get_messages.return_value = msgs
        q = self.mock_sqs.queue.Queue()
        q_name = 'mysqs'
        mock_fire = MagicMock(return_value=True)
        sqs_events._process_queue(q, q_name, mock_fire)
        self.assertTrue(self.mock_sqs.queue.Queue().get_messages.called, len(msgs))
        self.assertTrue(mock_fire.called, len(msgs))



# -*- coding: utf-8 -*-
'''
    :codeauthor: Pedro Algarvio (pedro@algarvio.me)
    :codeauthor: Alexandru Bleotu (alexandru.bleotu@morganstanley.com)

    tests.unit.pillar_test
    ~~~~~~~~~~~~~~~~~~~~~~
'''

# Import python libs
from __future__ import absolute_import
import tempfile

# Import Salt Testing libs
from tests.support.unit import skipIf, TestCase
from tests.support.mock import NO_MOCK, NO_MOCK_REASON, MagicMock, patch
from tests.support.paths import TMP

# Import salt libs
import salt.pillar
import salt.utils.stringutils
import salt.exceptions


@skipIf(NO_MOCK, NO_MOCK_REASON)
class PillarTestCase(TestCase):

    def tearDown(self):
        for attrname in ('generic_file', 'generic_minion_file', 'ssh_file', 'ssh_minion_file', 'top_file'):
            try:
                delattr(self, attrname)
            except AttributeError:
                continue

    def test_pillarenv_from_saltenv(self):
        with patch('salt.pillar.compile_template') as compile_template:
            opts = {
                'optimization_order': [0, 1, 2],
                'renderer': 'json',
                'renderer_blacklist': [],
                'renderer_whitelist': [],
                'state_top': '',
                'pillar_roots': {
                    'dev': [],
                    'base': []
                },
                'file_roots': {
                    'dev': [],
                    'base': []
                },
                'extension_modules': '',
                'pillarenv_from_saltenv': True
            }
            grains = {
                'os': 'Ubuntu',
            }
            pillar = salt.pillar.Pillar(opts, grains, 'mocked-minion', 'dev')
            self.assertEqual(pillar.opts['saltenv'], 'dev')
            self.assertEqual(pillar.opts['pillarenv'], 'dev')

    def test_ext_pillar_no_extra_minion_data_val_dict(self):
        opts = {
            'optimization_order': [0, 1, 2],
            'renderer': 'json',
            'renderer_blacklist': [],
            'renderer_whitelist': [],
            'state_top': '',
            'pillar_roots': {
                'dev': [],
                'base': []
            },
            'file_roots': {
                'dev': [],
                'base': []
            },
            'extension_modules': '',
            'pillarenv_from_saltenv': True
        }
        mock_ext_pillar_func = MagicMock()
        with patch('salt.loader.pillars',
                   MagicMock(return_value={'fake_ext_pillar':
                                           mock_ext_pillar_func})):
            pillar = salt.pillar.Pillar(opts, {}, 'mocked-minion', 'dev')
        # ext pillar function doesn't have the extra_minion_data arg
        with patch('salt.utils.args.get_function_argspec',
                   MagicMock(return_value=MagicMock(args=[]))):
            pillar._external_pillar_data('fake_pillar', {'arg': 'foo'},
                                         'fake_ext_pillar')
        mock_ext_pillar_func.assert_called_once_with('mocked-minion',
                                                     'fake_pillar',
                                                     arg='foo')
        # ext pillar function has the extra_minion_data arg
        mock_ext_pillar_func.reset_mock()
        with patch('salt.utils.args.get_function_argspec',
                   MagicMock(return_value=MagicMock(args=['extra_minion_data']))):
            pillar._external_pillar_data('fake_pillar', {'arg': 'foo'},
                                         'fake_ext_pillar')
        mock_ext_pillar_func.assert_called_once_with('mocked-minion',
                                                     'fake_pillar',
                                                     arg='foo')

    def test_ext_pillar_no_extra_minion_data_val_list(self):
        opts = {
            'optimization_order': [0, 1, 2],
            'renderer': 'json',
            'renderer_blacklist': [],
            'renderer_whitelist': [],
            'state_top': '',
            'pillar_roots': {
                'dev': [],
                'base': []
            },
            'file_roots': {
                'dev': [],
                'base': []
            },
            'extension_modules': '',
            'pillarenv_from_saltenv': True
        }
        mock_ext_pillar_func = MagicMock()
        with patch('salt.loader.pillars',
                   MagicMock(return_value={'fake_ext_pillar':
                                           mock_ext_pillar_func})):
            pillar = salt.pillar.Pillar(opts, {}, 'mocked-minion', 'dev')
        # ext pillar function doesn't have the extra_minion_data arg
        with patch('salt.utils.args.get_function_argspec',
                   MagicMock(return_value=MagicMock(args=[]))):
            pillar._external_pillar_data('fake_pillar', ['foo'],
                                         'fake_ext_pillar')
        mock_ext_pillar_func.assert_called_once_with('mocked-minion',
                                                     'fake_pillar',
                                                     'foo')
        # ext pillar function has the extra_minion_data arg
        mock_ext_pillar_func.reset_mock()
        with patch('salt.utils.args.get_function_argspec',
                   MagicMock(return_value=MagicMock(args=['extra_minion_data']))):
            pillar._external_pillar_data('fake_pillar', ['foo'],
                                         'fake_ext_pillar')
        mock_ext_pillar_func.assert_called_once_with('mocked-minion',
                                                     'fake_pillar',
                                                     'foo')

    def test_ext_pillar_no_extra_minion_data_val_elem(self):
        opts = {
            'optimization_order': [0, 1, 2],
            'renderer': 'json',
            'renderer_blacklist': [],
            'renderer_whitelist': [],
            'state_top': '',
            'pillar_roots': {
                'dev': [],
                'base': []
            },
            'file_roots': {
                'dev': [],
                'base': []
            },
            'extension_modules': '',
            'pillarenv_from_saltenv': True
        }
        mock_ext_pillar_func = MagicMock()
        with patch('salt.loader.pillars',
                   MagicMock(return_value={'fake_ext_pillar':
                                           mock_ext_pillar_func})):
            pillar = salt.pillar.Pillar(opts, {}, 'mocked-minion', 'dev')
        # ext pillar function doesn't have the extra_minion_data arg
        with patch('salt.utils.args.get_function_argspec',
                   MagicMock(return_value=MagicMock(args=[]))):
            pillar._external_pillar_data('fake_pillar', 'fake_val',
                                         'fake_ext_pillar')
        mock_ext_pillar_func.assert_called_once_with('mocked-minion',
                                                     'fake_pillar', 'fake_val')
        # ext pillar function has the extra_minion_data arg
        mock_ext_pillar_func.reset_mock()
        with patch('salt.utils.args.get_function_argspec',
                   MagicMock(return_value=MagicMock(args=['extra_minion_data']))):
            pillar._external_pillar_data('fake_pillar', 'fake_val',
                                         'fake_ext_pillar')
        mock_ext_pillar_func.assert_called_once_with('mocked-minion',
                                                     'fake_pillar', 'fake_val')

    def test_ext_pillar_with_extra_minion_data_val_dict(self):
        opts = {
            'optimization_order': [0, 1, 2],
            'renderer': 'json',
            'renderer_blacklist': [],
            'renderer_whitelist': [],
            'state_top': '',
            'pillar_roots': {
                'dev': [],
                'base': []
            },
            'file_roots': {
                'dev': [],
                'base': []
            },
            'extension_modules': '',
            'pillarenv_from_saltenv': True
        }
        mock_ext_pillar_func = MagicMock()
        with patch('salt.loader.pillars',
                   MagicMock(return_value={'fake_ext_pillar':
                                           mock_ext_pillar_func})):
            pillar = salt.pillar.Pillar(opts, {}, 'mocked-minion', 'dev',
                                        extra_minion_data={'fake_key': 'foo'})
        # ext pillar function doesn't have the extra_minion_data arg
        with patch('salt.utils.args.get_function_argspec',
                   MagicMock(return_value=MagicMock(args=[]))):
            pillar._external_pillar_data('fake_pillar', {'arg': 'foo'},
                                         'fake_ext_pillar')
        mock_ext_pillar_func.assert_called_once_with(
            'mocked-minion', 'fake_pillar', arg='foo')
        # ext pillar function has the extra_minion_data arg
        mock_ext_pillar_func.reset_mock()
        with patch('salt.utils.args.get_function_argspec',
                   MagicMock(return_value=MagicMock(args=['extra_minion_data']))):
            pillar._external_pillar_data('fake_pillar', {'arg': 'foo'},
                                         'fake_ext_pillar')
        mock_ext_pillar_func.assert_called_once_with(
            'mocked-minion', 'fake_pillar', arg='foo',
            extra_minion_data={'fake_key': 'foo'})

    def test_ext_pillar_with_extra_minion_data_val_list(self):
        opts = {
            'optimization_order': [0, 1, 2],
            'renderer': 'json',
            'renderer_blacklist': [],
            'renderer_whitelist': [],
            'state_top': '',
            'pillar_roots': {
                'dev': [],
                'base': []
            },
            'file_roots': {
                'dev': [],
                'base': []
            },
            'extension_modules': '',
            'pillarenv_from_saltenv': True
        }
        mock_ext_pillar_func = MagicMock()
        with patch('salt.loader.pillars',
                   MagicMock(return_value={'fake_ext_pillar':
                                           mock_ext_pillar_func})):
            pillar = salt.pillar.Pillar(opts, {}, 'mocked-minion', 'dev',
                                        extra_minion_data={'fake_key': 'foo'})
        # ext pillar function doesn't have the extra_minion_data arg
        with patch('salt.utils.args.get_function_argspec',
                   MagicMock(return_value=MagicMock(args=[]))):
            pillar._external_pillar_data('fake_pillar', ['bar'],
                                         'fake_ext_pillar')
        mock_ext_pillar_func.assert_called_once_with(
            'mocked-minion', 'fake_pillar', 'bar')
        # ext pillar function has the extra_minion_data arg
        mock_ext_pillar_func.reset_mock()
        with patch('salt.utils.args.get_function_argspec',
                   MagicMock(return_value=MagicMock(args=['extra_minion_data']))):
            pillar._external_pillar_data('fake_pillar', ['bar'],
                                         'fake_ext_pillar')
        mock_ext_pillar_func.assert_called_once_with(
            'mocked-minion', 'fake_pillar', 'bar',
            extra_minion_data={'fake_key': 'foo'})

    def test_ext_pillar_with_extra_minion_data_val_elem(self):
        opts = {
            'optimization_order': [0, 1, 2],
            'renderer': 'json',
            'renderer_blacklist': [],
            'renderer_whitelist': [],
            'state_top': '',
            'pillar_roots': {
                'dev': [],
                'base': []
            },
            'file_roots': {
                'dev': [],
                'base': []
            },
            'extension_modules': '',
            'pillarenv_from_saltenv': True
        }
        mock_ext_pillar_func = MagicMock()
        with patch('salt.loader.pillars',
                   MagicMock(return_value={'fake_ext_pillar':
                                           mock_ext_pillar_func})):
            pillar = salt.pillar.Pillar(opts, {}, 'mocked-minion', 'dev',
                                        extra_minion_data={'fake_key': 'foo'})
        # ext pillar function doesn't have the extra_minion_data arg
        with patch('salt.utils.args.get_function_argspec',
                   MagicMock(return_value=MagicMock(args=[]))):
            pillar._external_pillar_data('fake_pillar', 'bar',
                                         'fake_ext_pillar')
        mock_ext_pillar_func.assert_called_once_with(
            'mocked-minion', 'fake_pillar', 'bar')
        # ext pillar function has the extra_minion_data arg
        mock_ext_pillar_func.reset_mock()
        with patch('salt.utils.args.get_function_argspec',
                   MagicMock(return_value=MagicMock(args=['extra_minion_data']))):
            pillar._external_pillar_data('fake_pillar', 'bar',
                                         'fake_ext_pillar')
        mock_ext_pillar_func.assert_called_once_with(
            'mocked-minion', 'fake_pillar', 'bar',
            extra_minion_data={'fake_key': 'foo'})

    def test_dynamic_pillarenv(self):
        opts = {
            'optimization_order': [0, 1, 2],
            'renderer': 'json',
            'renderer_blacklist': [],
            'renderer_whitelist': [],
            'state_top': '',
            'pillar_roots': {'__env__': ['/srv/pillar/__env__'], 'base': ['/srv/pillar/base']},
            'file_roots': {'base': ['/srv/salt/base'], 'dev': ['/svr/salt/dev']},
            'extension_modules': '',
        }
        pillar = salt.pillar.Pillar(opts, {}, 'mocked-minion', 'base', pillarenv='dev')
        self.assertEqual(pillar.opts['pillar_roots'],
                         {'base': ['/srv/pillar/base'], 'dev': ['/srv/pillar/__env__']})

    def test_ignored_dynamic_pillarenv(self):
        opts = {
            'optimization_order': [0, 1, 2],
            'renderer': 'json',
            'renderer_blacklist': [],
            'renderer_whitelist': [],
            'state_top': '',
            'pillar_roots': {'__env__': ['/srv/pillar/__env__'], 'base': ['/srv/pillar/base']},
            'file_roots': {'base': ['/srv/salt/base'], 'dev': ['/svr/salt/dev']},
            'extension_modules': '',
        }
        pillar = salt.pillar.Pillar(opts, {}, 'mocked-minion', 'base', pillarenv='base')
        self.assertEqual(pillar.opts['pillar_roots'], {'base': ['/srv/pillar/base']})

    def test_malformed_pillar_sls(self):
        with patch('salt.pillar.compile_template') as compile_template:
            opts = {
                'optimization_order': [0, 1, 2],
                'renderer': 'json',
                'renderer_blacklist': [],
                'renderer_whitelist': [],
                'state_top': '',
                'pillar_roots': [],
                'file_roots': [],
                'extension_modules': ''
            }
            grains = {
                'os': 'Ubuntu',
                'os_family': 'Debian',
                'oscodename': 'raring',
                'osfullname': 'Ubuntu',
                'osrelease': '13.04',
                'kernel': 'Linux'
            }
            pillar = salt.pillar.Pillar(opts, grains, 'mocked-minion', 'base')
            # Mock getting the proper template files
            pillar.client.get_state = MagicMock(
                return_value={
                    'dest': '/path/to/pillar/files/foo.sls',
                    'source': 'salt://foo.sls'
                }
            )

            # Template compilation returned a string
            compile_template.return_value = 'BAHHH'
            self.assertEqual(
                pillar.render_pillar({'base': ['foo.sls']}),
                ({}, ['SLS \'foo.sls\' does not render to a dictionary'])
            )

            # Template compilation returned a list
            compile_template.return_value = ['BAHHH']
            self.assertEqual(
                pillar.render_pillar({'base': ['foo.sls']}),
                ({}, ['SLS \'foo.sls\' does not render to a dictionary'])
            )

            # Template compilation returned a dictionary, which is what's expected
            compile_template.return_value = {'foo': 'bar'}
            self.assertEqual(
                pillar.render_pillar({'base': ['foo.sls']}),
                ({'foo': 'bar'}, [])
            )

            # Test improper includes
            compile_template.side_effect = [
                {'foo': 'bar', 'include': 'blah'},
                {'foo2': 'bar2'}
            ]
            self.assertEqual(
                pillar.render_pillar({'base': ['foo.sls']}),
                ({'foo': 'bar', 'include': 'blah'},
                 ["Include Declaration in SLS 'foo.sls' is not formed as a list"])
            )

            # Test includes as a list, which is what's expected
            compile_template.side_effect = [
                {'foo': 'bar', 'include': ['blah']},
                {'foo2': 'bar2'}
            ]
            self.assertEqual(
                pillar.render_pillar({'base': ['foo.sls']}),
                ({'foo': 'bar', 'foo2': 'bar2'}, [])
            )

            # Test includes as a list overriding data
            compile_template.side_effect = [
                {'foo': 'bar', 'include': ['blah']},
                {'foo': 'bar2'}
            ]
            self.assertEqual(
                pillar.render_pillar({'base': ['foo.sls']}),
                ({'foo': 'bar'}, [])
            )

            # Test includes using empty key directive
            compile_template.side_effect = [
                {'foo': 'bar', 'include': [{'blah': {'key': ''}}]},
                {'foo': 'bar2'}
            ]
            self.assertEqual(
                pillar.render_pillar({'base': ['foo.sls']}),
                ({'foo': 'bar'}, [])
            )

            # Test includes using simple non-nested key
            compile_template.side_effect = [
                {'foo': 'bar', 'include': [{'blah': {'key': 'nested'}}]},
                {'foo': 'bar2'}
            ]
            self.assertEqual(
                pillar.render_pillar({'base': ['foo.sls']}),
                ({'foo': 'bar', 'nested': {'foo': 'bar2'}}, [])
            )

            # Test includes using nested key
            compile_template.side_effect = [
                {'foo': 'bar', 'include': [{'blah': {'key': 'nested:level'}}]},
                {'foo': 'bar2'}
            ]
            self.assertEqual(
                pillar.render_pillar({'base': ['foo.sls']}),
                ({'foo': 'bar', 'nested': {'level': {'foo': 'bar2'}}}, [])
            )

    def test_includes_override_sls(self):
        opts = {
            'optimization_order': [0, 1, 2],
            'renderer': 'json',
            'renderer_blacklist': [],
            'renderer_whitelist': [],
            'state_top': '',
            'pillar_roots': {},
            'file_roots': {},
            'extension_modules': ''
        }
        grains = {
            'os': 'Ubuntu',
            'os_family': 'Debian',
            'oscodename': 'raring',
            'osfullname': 'Ubuntu',
            'osrelease': '13.04',
            'kernel': 'Linux'
        }
        with patch('salt.pillar.compile_template') as compile_template:

            # Test with option set to True
            opts['pillar_includes_override_sls'] = True
            pillar = salt.pillar.Pillar(opts, grains, 'mocked-minion', 'base')
            # Mock getting the proper template files
            pillar.client.get_state = MagicMock(
                return_value={
                    'dest': '/path/to/pillar/files/foo.sls',
                    'source': 'salt://foo.sls'
                }
            )

            compile_template.side_effect = [
                {'foo': 'bar', 'include': ['blah']},
                {'foo': 'bar2'}
            ]
            self.assertEqual(
                pillar.render_pillar({'base': ['foo.sls']}),
                ({'foo': 'bar2'}, [])
            )

            # Test with option set to False
            opts['pillar_includes_override_sls'] = False
            pillar = salt.pillar.Pillar(opts, grains, 'mocked-minion', 'base')
            # Mock getting the proper template files
            pillar.client.get_state = MagicMock(
                return_value={
                    'dest': '/path/to/pillar/files/foo.sls',
                    'source': 'salt://foo.sls'
                }
            )

            compile_template.side_effect = [
                {'foo': 'bar', 'include': ['blah']},
                {'foo': 'bar2'}
            ]
            self.assertEqual(
                pillar.render_pillar({'base': ['foo.sls']}),
                ({'foo': 'bar'}, [])
            )

    def test_topfile_order(self):
        with patch('salt.pillar.salt.fileclient.get_file_client', autospec=True) as get_file_client, \
                patch('salt.pillar.salt.minion.Matcher') as Matcher:  # autospec=True disabled due to py3 mock bug
            opts = {
                'optimization_order': [0, 1, 2],
                'renderer': 'yaml',
                'renderer_blacklist': [],
                'renderer_whitelist': [],
                'state_top': '',
                'pillar_roots': [],
                'extension_modules': '',
                'saltenv': 'base',
                'file_roots': [],
            }
            grains = {
                'os': 'Ubuntu',
                'os_family': 'Debian',
                'oscodename': 'raring',
                'osfullname': 'Ubuntu',
                'osrelease': '13.04',
                'kernel': 'Linux'
            }
            # glob match takes precedence
            self._setup_test_topfile_mocks(Matcher, get_file_client, 1, 2)
            pillar = salt.pillar.Pillar(opts, grains, 'mocked-minion', 'base')
            self.assertEqual(pillar.compile_pillar()['ssh'], 'bar')
            # nodegroup match takes precedence
            self._setup_test_topfile_mocks(Matcher, get_file_client, 2, 1)
            pillar = salt.pillar.Pillar(opts, grains, 'mocked-minion', 'base')
            self.assertEqual(pillar.compile_pillar()['ssh'], 'foo')

    def _setup_test_topfile_mocks(self, Matcher, get_file_client,
                                  nodegroup_order, glob_order):
        # Write a simple topfile and two pillar state files
        self.top_file = tempfile.NamedTemporaryFile(dir=TMP, delete=False)
        s = '''
base:
    group:
        - match: nodegroup
        - order: {nodegroup_order}
        - ssh
        - generic
    '*':
        - generic
    minion:
        - order: {glob_order}
        - ssh.minion
        - generic.minion
'''.format(nodegroup_order=nodegroup_order, glob_order=glob_order)
        self.top_file.write(salt.utils.stringutils.to_bytes(s))
        self.top_file.flush()
        self.ssh_file = tempfile.NamedTemporaryFile(dir=TMP, delete=False)
        self.ssh_file.write(b'''
ssh:
    foo
''')
        self.ssh_file.flush()
        self.ssh_minion_file = tempfile.NamedTemporaryFile(dir=TMP, delete=False)
        self.ssh_minion_file.write(b'''
ssh:
    bar
''')
        self.ssh_minion_file.flush()
        self.generic_file = tempfile.NamedTemporaryFile(dir=TMP, delete=False)
        self.generic_file.write(b'''
generic:
    key1:
      - value1
      - value2
    key2:
        sub_key1: []
''')
        self.generic_file.flush()
        self.generic_minion_file = tempfile.NamedTemporaryFile(dir=TMP, delete=False)
        self.generic_minion_file.write(b'''
generic:
    key1:
      - value3
    key2:
        sub_key2: []
''')
        self.generic_minion_file.flush()

        # Setup Matcher mock
        matcher = Matcher.return_value
        matcher.confirm_top.return_value = True

        # Setup fileclient mock
        client = get_file_client.return_value
        client.cache_file.return_value = self.top_file.name

        def get_state(sls, env):
            return {
                'ssh': {'path': '', 'dest': self.ssh_file.name},
                'ssh.minion': {'path': '', 'dest': self.ssh_minion_file.name},
                'generic': {'path': '', 'dest': self.generic_file.name},
                'generic.minion': {'path': '', 'dest': self.generic_minion_file.name},
            }[sls]

        client.get_state.side_effect = get_state

    def _setup_test_include_mocks(self, Matcher, get_file_client):
        self.top_file = top_file = tempfile.NamedTemporaryFile(dir=TMP, delete=False)
        top_file.write(b'''
base:
    '*':
        - order: 1
        - test.sub2
    minion:
        - order: 2
        - test
''')
        top_file.flush()
        self.init_sls = init_sls = tempfile.NamedTemporaryFile(dir=TMP, delete=False)
        init_sls.write(b'''
include:
   - test.sub1
   - test.sub2
''')
        init_sls.flush()
        self.sub1_sls = sub1_sls = tempfile.NamedTemporaryFile(dir=TMP, delete=False)
        sub1_sls.write(b'''
p1:
   - value1_1
   - value1_2
''')
        sub1_sls.flush()
        self.sub2_sls = sub2_sls = tempfile.NamedTemporaryFile(dir=TMP, delete=False)
        sub2_sls.write(b'''
p1:
   - value1_3
p2:
   - value2_1
   - value2_2
''')
        sub2_sls.flush()

        # Setup Matcher mock
        matcher = Matcher.return_value
        matcher.confirm_top.return_value = True

        # Setup fileclient mock
        client = get_file_client.return_value
        client.cache_file.return_value = self.top_file.name

        def get_state(sls, env):
            return {
                'test': {'path': '', 'dest': init_sls.name},
                'test.sub1': {'path': '', 'dest': sub1_sls.name},
                'test.sub2': {'path': '', 'dest': sub2_sls.name},
            }[sls]

        client.get_state.side_effect = get_state


@skipIf(NO_MOCK, NO_MOCK_REASON)
@patch('salt.transport.Channel.factory', MagicMock())
class RemotePillarTestCase(TestCase):
    '''
    Tests for instantiating a RemotePillar in salt.pillar
    '''
    def setUp(self):
        self.grains = {}

    def tearDown(self):
        for attr in ('grains',):
            try:
                delattr(self, attr)
            except AttributeError:
                continue

    def test_get_opts_in_pillar_override_call(self):
        mock_get_extra_minion_data = MagicMock(return_value={})
        with patch(
            'salt.pillar.RemotePillarMixin.get_ext_pillar_extra_minion_data',
            mock_get_extra_minion_data):

            salt.pillar.RemotePillar({}, self.grains, 'mocked-minion', 'dev')
        mock_get_extra_minion_data.assert_called_once_with(
            {'saltenv': 'dev'})

    def test_multiple_keys_in_opts_added_to_pillar(self):
        opts = {
            'renderer': 'json',
            'path_to_add': 'fake_data',
            'path_to_add2': {'fake_data2': ['fake_data3', 'fake_data4']},
            'pass_to_ext_pillars': ['path_to_add', 'path_to_add2']
        }
        pillar = salt.pillar.RemotePillar(opts, self.grains,
                                          'mocked-minion', 'dev')
        self.assertEqual(pillar.extra_minion_data,
                         {'path_to_add': 'fake_data',
                          'path_to_add2': {'fake_data2': ['fake_data3',
                                                          'fake_data4']}})

    def test_subkey_in_opts_added_to_pillar(self):
        opts = {
            'renderer': 'json',
            'path_to_add': 'fake_data',
            'path_to_add2': {'fake_data5': 'fake_data6',
                             'fake_data2': ['fake_data3', 'fake_data4']},
            'pass_to_ext_pillars': ['path_to_add2:fake_data5']
        }
        pillar = salt.pillar.RemotePillar(opts, self.grains,
                                          'mocked-minion', 'dev')
        self.assertEqual(pillar.extra_minion_data,
                         {'path_to_add2': {'fake_data5': 'fake_data6'}})

    def test_non_existent_leaf_opt_in_add_to_pillar(self):
        opts = {
            'renderer': 'json',
            'path_to_add': 'fake_data',
            'path_to_add2': {'fake_data5': 'fake_data6',
                             'fake_data2': ['fake_data3', 'fake_data4']},
            'pass_to_ext_pillars': ['path_to_add2:fake_data_non_exist']
        }
        pillar = salt.pillar.RemotePillar(opts, self.grains,
                                          'mocked-minion', 'dev')
        self.assertEqual(pillar.pillar_override, {})

    def test_non_existent_intermediate_opt_in_add_to_pillar(self):
        opts = {
            'renderer': 'json',
            'path_to_add': 'fake_data',
            'path_to_add2': {'fake_data5': 'fake_data6',
                             'fake_data2': ['fake_data3', 'fake_data4']},
            'pass_to_ext_pillars': ['path_to_add_no_exist']
        }
        pillar = salt.pillar.RemotePillar(opts, self.grains,
                                          'mocked-minion', 'dev')
        self.assertEqual(pillar.pillar_override, {})

    def test_malformed_add_to_pillar(self):
        opts = {
            'renderer': 'json',
            'path_to_add': 'fake_data',
            'path_to_add2': {'fake_data5': 'fake_data6',
                             'fake_data2': ['fake_data3', 'fake_data4']},
            'pass_to_ext_pillars': MagicMock()
        }
        with self.assertRaises(salt.exceptions.SaltClientError) as excinfo:
            salt.pillar.RemotePillar(opts, self.grains, 'mocked-minion', 'dev')
        self.assertEqual(excinfo.exception.strerror,
                         '\'pass_to_ext_pillars\' config is malformed.')

    def test_pillar_send_extra_minion_data_from_config(self):
        opts = {
            'renderer': 'json',
            'pillarenv': 'fake_pillar_env',
            'path_to_add': 'fake_data',
            'path_to_add2': {'fake_data5': 'fake_data6',
                             'fake_data2': ['fake_data3', 'fake_data4']},
            'pass_to_ext_pillars': ['path_to_add']}
        mock_channel = MagicMock(
            crypted_transfer_decode_dictentry=MagicMock(return_value={}))
        with patch('salt.transport.Channel.factory',
                   MagicMock(return_value=mock_channel)):
            pillar = salt.pillar.RemotePillar(opts, self.grains,
                                              'mocked_minion', 'fake_env')

        ret = pillar.compile_pillar()
        self.assertEqual(pillar.channel, mock_channel)
        mock_channel.crypted_transfer_decode_dictentry.assert_called_once_with(
            {'cmd': '_pillar', 'ver': '2',
             'id': 'mocked_minion',
             'grains': {},
             'saltenv': 'fake_env',
             'pillarenv': 'fake_pillar_env',
             'pillar_override': {},
             'extra_minion_data': {'path_to_add': 'fake_data'}},
            dictkey='pillar')


@skipIf(NO_MOCK, NO_MOCK_REASON)
@patch('salt.transport.client.AsyncReqChannel.factory', MagicMock())
class AsyncRemotePillarTestCase(TestCase):
    '''
    Tests for instantiating a AsyncRemotePillar in salt.pillar
    '''
    def setUp(self):
        self.grains = {}

    def tearDown(self):
        for attr in ('grains',):
            try:
                delattr(self, attr)
            except AttributeError:
                continue

    def test_get_opts_in_pillar_override_call(self):
        mock_get_extra_minion_data = MagicMock(return_value={})
        with patch(
            'salt.pillar.RemotePillarMixin.get_ext_pillar_extra_minion_data',
            mock_get_extra_minion_data):

            salt.pillar.RemotePillar({}, self.grains, 'mocked-minion', 'dev')
        mock_get_extra_minion_data.assert_called_once_with(
            {'saltenv': 'dev'})

    def test_pillar_send_extra_minion_data_from_config(self):
        opts = {
            'renderer': 'json',
            'pillarenv': 'fake_pillar_env',
            'path_to_add': 'fake_data',
            'path_to_add2': {'fake_data5': 'fake_data6',
                             'fake_data2': ['fake_data3', 'fake_data4']},
            'pass_to_ext_pillars': ['path_to_add']}
        mock_channel = MagicMock(
            crypted_transfer_decode_dictentry=MagicMock(return_value={}))
        with patch('salt.transport.client.AsyncReqChannel.factory',
                   MagicMock(return_value=mock_channel)):
            pillar = salt.pillar.RemotePillar(opts, self.grains,
                                              'mocked_minion', 'fake_env')

        ret = pillar.compile_pillar()
        mock_channel.crypted_transfer_decode_dictentry.assert_called_once_with(
            {'cmd': '_pillar', 'ver': '2',
             'id': 'mocked_minion',
             'grains': {},
             'saltenv': 'fake_env',
             'pillarenv': 'fake_pillar_env',
             'pillar_override': {},
             'extra_minion_data': {'path_to_add': 'fake_data'}},
            dictkey='pillar')



# -*- coding: utf-8 -*-
'''
salt-ssh testing
'''
# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals
import os
import shutil

# Import salt testing libs
from tests.support.case import SSHCase


class SSHTest(SSHCase):
    '''
    Test general salt-ssh functionality
    '''
    def test_ping(self):
        '''
        Test a simple ping
        '''
        ret = self.run_function('test.ping')
        self.assertTrue(ret, 'Ping did not return true')

    def test_thin_dir(self):
        '''
        test to make sure thin_dir is created
        and salt-call file is included
        '''
        thin_dir = self.run_function('config.get', ['thin_dir'], wipe=False)
        os.path.isdir(thin_dir)
        os.path.exists(os.path.join(thin_dir, 'salt-call'))
        os.path.exists(os.path.join(thin_dir, 'running_data'))

    def tearDown(self):
        '''
        make sure to clean up any old ssh directories
        '''
        salt_dir = self.run_function('config.get', ['thin_dir'], wipe=False)
        if os.path.exists(salt_dir):
            shutil.rmtree(salt_dir)



# -*- coding: utf-8 -*-
'''
    :codeauthor: :email:`Jayesh Kariya <jayeshk@saltstack.com>`
'''

# Import Python Libs
from __future__ import absolute_import, print_function, unicode_literals
import os

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.paths import TMP
from tests.support.unit import TestCase, skipIf
from tests.support.mock import (
    MagicMock,
    patch,
    NO_MOCK,
    NO_MOCK_REASON
)

# Import Salt Libs
import salt.modules.beacons as beacons
from salt.utils.event import SaltEvent

SOCK_DIR = os.path.join(TMP, 'test-socks')


@skipIf(NO_MOCK, NO_MOCK_REASON)
class BeaconsTestCase(TestCase, LoaderModuleMockMixin):
    '''
    Test cases for salt.modules.beacons
    '''
    def setup_loader_modules(self):
        return {beacons: {}}

    def test_delete(self):
        '''
        Test deleting a beacon.
        '''
        comm1 = 'Deleted beacon: ps.'
        event_returns = [
                         {'complete': True,
                          'tag': '/salt/minion/minion_beacons_delete_complete',
                          'beacons': {}},
                        ]

        with patch.dict(beacons.__opts__, {'beacons': {'ps': [{'processes': {'salt-master': 'stopped', 'apache2': 'stopped'}}]}, 'sock_dir': SOCK_DIR}):
            mock = MagicMock(return_value=True)
            with patch.dict(beacons.__salt__, {'event.fire': mock}):
                with patch.object(SaltEvent, 'get_event', side_effect=event_returns):
                    self.assertDictEqual(beacons.delete('ps'),
                                         {'comment': comm1, 'result': True})

    def test_add(self):
        '''
        Test adding a beacon
        '''
        comm1 = 'Added beacon: ps.'
        event_returns = [{'complete': True,
                          'tag': '/salt/minion/minion_beacons_list_complete',
                          'beacons': {}},
                         {'complete': True,
                          'tag': '/salt/minion/minion_beacons_list_available_complete',
                          'beacons': ['ps']},
                         {'complete': True,
                          'valid': True,
                          'vcomment': '',
                          'tag': '/salt/minion/minion_beacons_list_complete'},
                         {'complete': True,
                          'tag': '/salt/minion/minion_beacon_add_complete',
                          'beacons': {'ps': [{'processes': {'salt-master': 'stopped', 'apache2': 'stopped'}}]}}]

        with patch.dict(beacons.__opts__, {'beacons': {}, 'sock_dir': SOCK_DIR}):
            mock = MagicMock(return_value=True)
            with patch.dict(beacons.__salt__, {'event.fire': mock}):
                with patch.object(SaltEvent, 'get_event', side_effect=event_returns):
                    self.assertDictEqual(beacons.add('ps', [{'processes': {'salt-master': 'stopped', 'apache2': 'stopped'}}]),
                                         {'comment': comm1, 'result': True})

    def test_save(self):
        '''
        Test saving beacons.
        '''
        comm1 = 'Beacons saved to {0}beacons.conf.'.format(TMP + os.sep)
        with patch.dict(beacons.__opts__, {'config_dir': '', 'beacons': {},
                                           'default_include': TMP + os.sep,
                                           'sock_dir': SOCK_DIR}):

            mock = MagicMock(return_value=True)
            with patch.dict(beacons.__salt__, {'event.fire': mock}):
                _ret_value = {'complete': True, 'beacons': {}}
                with patch.object(SaltEvent, 'get_event', return_value=_ret_value):
                    self.assertDictEqual(beacons.save(),
                                         {'comment': comm1, 'result': True})

    def test_disable(self):
        '''
        Test disabling beacons
        '''
        comm1 = 'Disabled beacons on minion.'
        event_returns = [{'complete': True,
                          'tag': '/salt/minion/minion_beacons_disabled_complete',
                          'beacons': {'enabled': False,
                                      'ps': [{'processes': {'salt-master': 'stopped',
                                                            'apache2': 'stopped'}}]}}]

        with patch.dict(beacons.__opts__, {'beacons': {}, 'sock_dir': SOCK_DIR}):
            mock = MagicMock(return_value=True)
            with patch.dict(beacons.__salt__, {'event.fire': mock}):
                with patch.object(SaltEvent, 'get_event', side_effect=event_returns):
                    self.assertDictEqual(beacons.disable(),
                                         {'comment': comm1, 'result': True})

    def test_enable(self):
        '''
        Test enabling beacons
        '''
        comm1 = 'Enabled beacons on minion.'
        event_returns = [{'complete': True,
                          'tag': '/salt/minion/minion_beacon_enabled_complete',
                          'beacons': {'enabled': True,
                                      'ps': [{'processes': {'salt-master': 'stopped',
                                                            'apache2': 'stopped'}}]}}]

        with patch.dict(beacons.__opts__, {'beacons': {}, 'sock_dir': SOCK_DIR}):
            mock = MagicMock(return_value=True)
            with patch.dict(beacons.__salt__, {'event.fire': mock}):
                with patch.object(SaltEvent, 'get_event', side_effect=event_returns):
                    self.assertDictEqual(beacons.enable(),
                                         {'comment': comm1, 'result': True})



# -*- coding: utf-8 -*-
'''
    :codeauthor: Gareth J. Greenaway <gareth@saltstack.com>
'''

# Import Python Libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import TestCase, skipIf
from tests.support.mock import (
    MagicMock,
    patch,
    NO_MOCK,
    NO_MOCK_REASON
)

# Import Salt Libs
import salt.modules.osquery as osquery


@skipIf(NO_MOCK, NO_MOCK_REASON)
class OSQueryTestCase(TestCase, LoaderModuleMockMixin):
    '''
    Test cases for salt.modules.iptables
    '''
    def setup_loader_modules(self):
        return {osquery: {}}

    def test_version(self):
        '''
        Test the version returned from OSQuery
        '''
        _table_attrs_results = ['pid',
                                'uuid',
                                'instance_id',
                                'version',
                                'config_hash',
                                'config_valid',
                                'extensions',
                                'build_platform',
                                'build_distro',
                                'start_time',
                                'watcher']

        _os_query_results = {'data': [{'version': '2.6.1'}], 'result': True}

        with patch.object(osquery, '_table_attrs',
                          MagicMock(return_value=_table_attrs_results)):
            with patch.object(osquery, '_osquery',
                              MagicMock(return_value=_os_query_results)):
                self.assertEqual(osquery.version(), '2.6.1')

    def test_deb_packages(self):
        '''
        Test the results returned from the deb_packages function
        '''
        _os_query_results = {'data': [
            {'arch': 'amd64', 'name': 'accountsservice', 'revision': '1',
             'size': '451', 'source': '', 'version': '0.6.45-1'},
            {'arch': 'amd64', 'name': 'acetoneiso', 'revision': '2+b2',
             'size': '1820', 'source': 'acetoneiso (2.4-2)',
             'version': '2.4-2+b2'},
            {'arch': 'amd64', 'name': 'acl', 'revision': '3+b1',
             'size': '200', 'source': 'acl (2.2.52-3)',
             'version': '2.2.52-3+b1'},
            {'arch': 'amd64', 'name': 'adb', 'revision': '2', 'size': '189',
             'source': 'android-platform-system-core',
             'version': '1: 7.0.0+r33-2'}],
            'result': True
        }
        with patch.object(osquery, '_osquery',
                          MagicMock(return_value=_os_query_results)):
            with patch.dict(osquery.__grains__, {'os_family': 'Debian'}):
                self.assertEqual(osquery.deb_packages(), _os_query_results)

    def test_deb_packages_with_attrs(self):
        '''
        Test the results returned from the deb_packages function
        with attributes
        '''
        _table_attrs_results = ['name',
                                'version',
                                'source',
                                'size',
                                'arch',
                                'revision']

        _os_query_results = {'data': [
            {'name': 'accountsservice', 'version': '0.6.45-1'},
            {'name': 'acetoneiso', 'version': '2.4-2+b2'},
            {'name': 'acl', 'version': '2.2.52-3+b1'},
            {'name': 'adb', 'version': '1: 7.0.0+r33-2'}],
            'result': True}

        with patch.object(osquery, '_table_attrs',
                          MagicMock(return_value=_table_attrs_results)):
            with patch.object(osquery, '_osquery',
                              MagicMock(return_value=_os_query_results)):
                with patch.dict(osquery.__grains__, {'os_family': 'Debian'}):
                    self.assertEqual(osquery.deb_packages(attrs=['name',
                                                                 'version']),
                                     _os_query_results)

    def test_kernel_modules(self):
        '''
        Test the results returned from the kernel_modules function
        '''
        _os_query_results = {'data': [
            {'address': '0xffffffffc14f2000', 'name': 'nls_utf8',
             'size': '16384', 'status': 'Live', 'used_by': '-'},
            {'address': '0xffffffffc1599000', 'name': 'udf',
             'size': '90112', 'status': 'Live', 'used_by': '-'},
            {'address': '0xffffffffc14b5000', 'name': 'crc_itu_t',
             'size': '16384', 'status': 'Live', 'used_by': 'udf'}],
            'result': True
        }
        with patch.object(osquery, '_osquery',
                          MagicMock(return_value=_os_query_results)):
            with patch.dict(osquery.__grains__, {'os_family': 'Debian'}):
                self.assertEqual(osquery.kernel_modules(),
                                 _os_query_results)

    def test_kernel_modules_with_attrs(self):
        '''
        Test the results returned from the kernel_modules function
        with attributes
        '''
        _table_attrs_results = ['address',
                                'name',
                                'size',
                                'status',
                                'used_by']

        _os_query_results = {'data': [
            {'name': 'nls_utf8', 'status': 'Live'},
            {'name': 'udf', 'status': 'Live'},
            {'name': 'crc_itu_t', 'status': 'Live'}],
            'result': True
        }
        with patch.object(osquery, '_table_attrs',
                          MagicMock(return_value=_table_attrs_results)):
            with patch.object(osquery, '_osquery',
                              MagicMock(return_value=_os_query_results)):
                with patch.dict(osquery.__grains__, {'os_family': 'Debian'}):
                    self.assertEqual(osquery.kernel_modules(attrs=['name',
                                                                   'status']),
                                     _os_query_results)

    def test_osquery_info(self):
        '''
        Test the results returned from the kernel_modules function
        with attributes
        '''
        _table_attrs_results = ['pid',
                                'uuid',
                                'instance_id',
                                'version',
                                'config_hash',
                                'config_valid',
                                'extensions',
                                'build_platform',
                                'build_distro',
                                'start_time',
                                'watcher']

        _os_query_results = {'data': [
            {'build_platform': 'ubuntu', 'start_time': '1514484833',
             'uuid': 'D31FD400-7277-11E3-ABA6-B8AEED7E173B',
             'build_distro': 'xenial',
             'pid': '24288',
             'watcher': '-1',
             'instance_id': 'dff196b0-5c91-4105-962b-28660d7aa282',
             'version': '2.6.1',
             'extensions': 'inactive',
             'config_valid': '0',
             'config_hash': ''}],
            'result': True}

        with patch.object(osquery, '_table_attrs',
                          MagicMock(return_value=_table_attrs_results)):
            with patch.object(osquery, '_osquery',
                              MagicMock(return_value=_os_query_results)):
                with patch.dict(osquery.__grains__, {'os_family': 'Debian'}):
                    self.assertEqual(osquery.osquery_info(),
                                     _os_query_results)

    def test_osquery_info_with_attrs(self):
        '''
        Test the results returned from the kernel_modules function
        with attributes
        '''
        _table_attrs_results = ['pid',
                                'uuid',
                                'instance_id',
                                'version',
                                'config_hash',
                                'config_valid',
                                'extensions',
                                'build_platform',
                                'build_distro',
                                'start_time',
                                'watcher']

        _os_query_results = {'data': [
            {'build_platform': 'ubuntu', 'start_time': '1514484833'}],
            'result': True}

        with patch.object(osquery, '_table_attrs',
                          MagicMock(return_value=_table_attrs_results)):
            with patch.object(osquery, '_osquery',
                              MagicMock(return_value=_os_query_results)):
                with patch.dict(osquery.__grains__, {'os_family': 'Debian'}):
                    self.assertEqual(osquery.osquery_info(attrs=['build_platform', 'start_time']),
                                     _os_query_results)



# -*- coding: utf-8 -*-

# Import python libs
from __future__ import absolute_import
import os

# Import Salt Testing libs
from tests.support.case import ShellCase, SPMCase


class SPMTest(ShellCase, SPMCase):
    '''
    Test spm script
    '''

    def test_spm_help(self):
        '''
        test --help argument for spm
        '''
        expected_args = ['--version', '--assume-yes', '--help']
        output = self.run_spm('--help')
        for arg in expected_args:
            self.assertIn(arg, ''.join(output))

    def test_spm_bad_arg(self):
        '''
        test correct output when bad argument passed
        '''
        expected_args = ['--version', '--assume-yes', '--help']
        output = self.run_spm('doesnotexist')
        for arg in expected_args:
            self.assertIn(arg, ''.join(output))

    def test_spm_assume_yes(self):
        '''
        test spm install with -y arg
        '''
        config = self._spm_config(assume_yes=False)
        self._spm_build_files(config)

        spm_file = os.path.join(config['spm_build_dir'],
                                'apache-201506-2.spm')

        build = self.run_spm('build {0} -c {1}'.format(self.formula_dir,
                                                       self._tmp_spm))

        install = self.run_spm('install {0} -c {1} -y'.format(spm_file,
                                                           self._tmp_spm))

        self.assertTrue(os.path.exists(os.path.join(config['formula_path'],
                                                    'apache', 'apache.sls')))

    def test_spm_force(self):
        '''
        test spm install with -f arg
        '''
        config = self._spm_config(assume_yes=False)
        self._spm_build_files(config)

        spm_file = os.path.join(config['spm_build_dir'],
                                'apache-201506-2.spm')

        build = self.run_spm('build {0} -c {1}'.format(self.formula_dir,
                                                       self._tmp_spm))

        install = self.run_spm('install {0} -c {1} -y'.format(spm_file,
                                                           self._tmp_spm))

        self.assertTrue(os.path.exists(os.path.join(config['formula_path'],
                                                    'apache', 'apache.sls')))

        # check if it forces the install after its already been installed it
        install = self.run_spm('install {0} -c {1} -y -f'.format(spm_file,
                                                           self._tmp_spm))

        self.assertEqual(['... installing apache'], install)



# -*- coding: utf-8 -*-
'''
Tests for the win_pkg module
'''

# Import Python Libs
from __future__ import absolute_import, unicode_literals, print_function

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase

# Import Salt Libs
import salt.modules.pkg_resource as pkg_resource
import salt.modules.win_pkg as win_pkg


class WinPkgInstallTestCase(TestCase, LoaderModuleMockMixin):
    '''
    Test cases for salt.modules.win_pkg
    '''
    def setup_loader_modules(self):
        pkg_info = {
            '3.03': {
                'full_name': 'Nullsoft Install System',
                'installer': 'http://download.sourceforge.net/project/nsis/NSIS%203/3.03/nsis-3.03-setup.exe',
                'install_flags': '/S',
                'uninstaller': '%PROGRAMFILES(x86)%\\NSIS\\uninst-nsis.exe',
                'uninstall_flags': '/S',
                'msiexec': False,
                'reboot': False
            },
            '3.02': {
                'full_name': 'Nullsoft Install System',
                'installer': 'http://download.sourceforge.net/project/nsis/NSIS%203/3.02/nsis-3.02-setup.exe',
                'install_flags': '/S',
                'uninstaller': '%PROGRAMFILES(x86)%\\NSIS\\uninst-nsis.exe',
                'uninstall_flags': '/S',
                'msiexec': False,
                'reboot': False
            }
        }

        return{
            win_pkg: {
                '_get_latest_package_version': MagicMock(return_value='3.03'),
                '_get_package_info': MagicMock(return_value=pkg_info),
                '__salt__': {
                    'pkg_resource.add_pkg': pkg_resource.add_pkg,
                    'pkg_resource.parse_targets': pkg_resource.parse_targets,
                    'pkg_resource.sort_pkglist': pkg_resource.sort_pkglist,
                    'pkg_resource.stringify': pkg_resource.stringify,
                },
            },
            pkg_resource: {
                '__grains__': {
                    'os': 'Windows'
                }
            },
        }

    def test_pkg_install_not_found(self):
        '''
        Test pkg.install when the Version is NOT FOUND in the Software
        Definition
        '''
        ret_reg = {'Nullsoft Install System': '3.03'}
        # The 2nd time it's run with stringify
        se_list_pkgs = {'nsis': ['3.03']}
        with patch.object(win_pkg, 'list_pkgs', return_value=se_list_pkgs), \
                patch.object(win_pkg, '_get_reg_software', return_value=ret_reg):
            expected = {'nsis': {'not found': '3.01'}}
            result = win_pkg.install(name='nsis', version='3.01')
            self.assertDictEqual(expected, result)

    def test_pkg_install_rollback(self):
        '''
        test pkg.install rolling back to a previous version
        '''
        ret_reg = {'Nullsoft Install System': '3.03'}
        # The 2nd time it's run, pkg.list_pkgs uses with stringify
        se_list_pkgs = [{'nsis': ['3.03']},
                        {'nsis': '3.02'}]
        with patch.object(win_pkg, 'list_pkgs', side_effect=se_list_pkgs), \
                patch.object(
                    win_pkg, '_get_reg_software', return_value=ret_reg), \
                patch.dict(
                    win_pkg.__salt__,
                    {'cp.is_cached': MagicMock(return_value=False)}), \
                patch.dict(
                    win_pkg.__salt__,
                    {'cp.cache_file':
                         MagicMock(return_value='C:\\fake\\path.exe')}), \
                patch.dict(
                    win_pkg.__salt__,
                    {'cmd.run_all':
                         MagicMock(return_value={'retcode': 0})}):
            expected = {'nsis': {'new': '3.02', 'old': '3.03'}}
            result = win_pkg.install(name='nsis', version='3.02')
            self.assertDictEqual(expected, result)

    def test_pkg_install_existing(self):
        '''
        test pkg.install when the package is already installed
        no version passed
        '''
        ret_reg = {'Nullsoft Install System': '3.03'}
        # The 2nd time it's run, pkg.list_pkgs uses with stringify
        se_list_pkgs = {'nsis': ['3.03']}
        with patch.object(win_pkg, 'list_pkgs', return_value=se_list_pkgs), \
                patch.object(
                    win_pkg, '_get_reg_software', return_value=ret_reg), \
                patch.dict(
                    win_pkg.__salt__,
                    {'cp.is_cached': MagicMock(return_value=False)}), \
                patch.dict(
                    win_pkg.__salt__,
                    {'cp.cache_file':
                         MagicMock(return_value='C:\\fake\\path.exe')}), \
                patch.dict(
                    win_pkg.__salt__,
                    {'cmd.run_all':
                         MagicMock(return_value={'retcode': 0})}):
            expected = {}
            result = win_pkg.install(name='nsis')
            self.assertDictEqual(expected, result)

    def test_pkg_install_existing_with_version(self):
        '''
        test pkg.install when the package is already installed
        A version is passed
        '''
        ret_reg = {'Nullsoft Install System': '3.03'}
        # The 2nd time it's run, pkg.list_pkgs uses with stringify
        se_list_pkgs = {'nsis': ['3.03']}
        with patch.object(win_pkg, 'list_pkgs', return_value=se_list_pkgs), \
                patch.object(
                    win_pkg, '_get_reg_software', return_value=ret_reg), \
                patch.dict(
                    win_pkg.__salt__,
                    {'cp.is_cached': MagicMock(return_value=False)}), \
                patch.dict(
                    win_pkg.__salt__,
                    {'cp.cache_file':
                         MagicMock(return_value='C:\\fake\\path.exe')}), \
                patch.dict(
                    win_pkg.__salt__,
                    {'cmd.run_all':
                         MagicMock(return_value={'retcode': 0})}):
            expected = {}
            result = win_pkg.install(name='nsis', version='3.03')
            self.assertDictEqual(expected, result)



# coding: utf-8

# Import python libs
from __future__ import absolute_import
import os

# Import salt libs
import salt.utils.json
import salt.utils.stringutils

# Import test support libs
import tests.support.cherrypy_testclasses as cptc

# Import 3rd-party libs
from salt.ext.six.moves.urllib.parse import urlencode  # pylint: disable=no-name-in-module,import-error


class TestAuth(cptc.BaseRestCherryPyTest):
    def test_get_root_noauth(self):
        '''
        GET requests to the root URL should not require auth
        '''
        request, response = self.request('/')
        self.assertEqual(response.status, '200 OK')

    def test_post_root_auth(self):
        '''
        POST requests to the root URL redirect to login
        '''
        request, response = self.request('/', method='POST', data={})
        self.assertEqual(response.status, '401 Unauthorized')

    def test_login_noauth(self):
        '''
        GET requests to the login URL should not require auth
        '''
        request, response = self.request('/login')
        self.assertEqual(response.status, '200 OK')

    def test_webhook_auth(self):
        '''
        Requests to the webhook URL require auth by default
        '''
        request, response = self.request('/hook', method='POST', data={})
        self.assertEqual(response.status, '401 Unauthorized')


class TestLogin(cptc.BaseRestCherryPyTest):
    auth_creds = (
            ('username', 'saltdev'),
            ('password', 'saltdev'),
            ('eauth', 'auto'))

    def test_good_login(self):
        '''
        Test logging in
        '''
        body = urlencode(self.auth_creds)
        request, response = self.request('/login', method='POST', body=body,
            headers={
                'content-type': 'application/x-www-form-urlencoded'
        })
        self.assertEqual(response.status, '200 OK')
        return response

    def test_bad_login(self):
        '''
        Test logging in
        '''
        body = urlencode({'totally': 'invalid_creds'})
        request, response = self.request('/login', method='POST', body=body,
            headers={
                'content-type': 'application/x-www-form-urlencoded'
        })
        self.assertEqual(response.status, '401 Unauthorized')

    def test_logout(self):
        ret = self.test_good_login()
        token = ret.headers['X-Auth-Token']

        body = urlencode({})
        request, response = self.request('/logout', method='POST', body=body,
            headers={
                'content-type': 'application/x-www-form-urlencoded',
                'X-Auth-Token': token,
        })
        self.assertEqual(response.status, '200 OK')


class TestRun(cptc.BaseRestCherryPyTest):
    auth_creds = (
        ('username', 'saltdev_auto'),
        ('password', 'saltdev'),
        ('eauth', 'auto'))

    low = (
        ('client', 'local'),
        ('tgt', '*'),
        ('fun', 'test.ping'),
    )

    def test_run_good_login(self):
        '''
        Test the run URL with good auth credentials
        '''
        cmd = dict(self.low, **dict(self.auth_creds))
        body = urlencode(cmd)

        request, response = self.request('/run', method='POST', body=body,
            headers={
                'content-type': 'application/x-www-form-urlencoded'
        })
        self.assertEqual(response.status, '200 OK')

    def test_run_bad_login(self):
        '''
        Test the run URL with bad auth credentials
        '''
        cmd = dict(self.low, **{'totally': 'invalid_creds'})
        body = urlencode(cmd)

        request, response = self.request('/run', method='POST', body=body,
            headers={
                'content-type': 'application/x-www-form-urlencoded'
        })
        self.assertEqual(response.status, '401 Unauthorized')

    def test_run_empty_token(self):
        '''
        Test the run URL with empty token
        '''
        cmd = dict(self.low, **{'token': ''})
        body = urlencode(cmd)

        request, response = self.request('/run', method='POST', body=body,
            headers={
                'content-type': 'application/x-www-form-urlencoded'
        })
        assert response.status == '401 Unauthorized'

    def test_run_empty_token_upercase(self):
        '''
        Test the run URL with empty token with upercase characters
        '''
        cmd = dict(self.low, **{'ToKen': ''})
        body = urlencode(cmd)

        request, response = self.request('/run', method='POST', body=body,
            headers={
                'content-type': 'application/x-www-form-urlencoded'
        })
        assert response.status == '401 Unauthorized'

    def test_run_wrong_token(self):
        '''
        Test the run URL with incorrect token
        '''
        cmd = dict(self.low, **{'token': 'bad'})
        body = urlencode(cmd)

        request, response = self.request('/run', method='POST', body=body,
            headers={
                'content-type': 'application/x-www-form-urlencoded'
        })
        assert response.status == '401 Unauthorized'

    def test_run_pathname_token(self):
        '''
        Test the run URL with path that exists in token
        '''
        cmd = dict(self.low, **{'token': os.path.join('etc', 'passwd')})
        body = urlencode(cmd)

        request, response = self.request('/run', method='POST', body=body,
            headers={
                'content-type': 'application/x-www-form-urlencoded'
        })
        assert response.status == '401 Unauthorized'

    def test_run_pathname_not_exists_token(self):
        '''
        Test the run URL with path that does not exist in token
        '''
        cmd = dict(self.low, **{'token': os.path.join('tmp', 'doesnotexist')})
        body = urlencode(cmd)

        request, response = self.request('/run', method='POST', body=body,
            headers={
                'content-type': 'application/x-www-form-urlencoded'
        })
        assert response.status == '401 Unauthorized'


class TestWebhookDisableAuth(cptc.BaseRestCherryPyTest):

    def __get_opts__(self):
        return {
            'rest_cherrypy': {
                'port': 8000,
                'debug': True,
                'webhook_disable_auth': True,
            },
        }

    def test_webhook_noauth(self):
        '''
        Auth can be disabled for requests to the webhook URL
        '''
        body = urlencode({'foo': 'Foo!'})
        request, response = self.request('/hook', method='POST', body=body,
            headers={
                'content-type': 'application/x-www-form-urlencoded'
        })
        self.assertEqual(response.status, '200 OK')


class TestArgKwarg(cptc.BaseRestCherryPyTest):
    auth_creds = (
        ('username', 'saltdev'),
        ('password', 'saltdev'),
        ('eauth', 'auto'))

    low = (
        ('client', 'runner'),
        ('fun', 'test.arg'),
        # use singular form for arg and kwarg
        ('arg', [1234]),
        ('kwarg', {'ext_source': 'redis'}),
    )

    def _token(self):
        '''
        Return the token
        '''
        body = urlencode(self.auth_creds)
        request, response = self.request(
            '/login',
            method='POST',
            body=body,
            headers={
                'content-type': 'application/x-www-form-urlencoded'
            }
        )
        return response.headers['X-Auth-Token']

    def test_accepts_arg_kwarg_keys(self):
        '''
        Ensure that (singular) arg and kwarg keys (for passing parameters)
        are supported by runners.
        '''
        cmd = dict(self.low)
        body = salt.utils.json.dumps(cmd)

        request, response = self.request(
            '/',
            method='POST',
            body=body,
            headers={
                'content-type': 'application/json',
                'X-Auth-Token': self._token(),
                'Accept': 'application/json',
            }
        )
        resp = salt.utils.json.loads(salt.utils.stringutils.to_str(response.body[0]))
        self.assertEqual(resp['return'][0]['args'], [1234])
        self.assertEqual(resp['return'][0]['kwargs'],
                         {'ext_source': 'redis'})


class TestJobs(cptc.BaseRestCherryPyTest):
    auth_creds = (
        ('username', 'saltdev_auto'),
        ('password', 'saltdev'),
        ('eauth', 'auto'))

    low = (
        ('client', 'local'),
        ('tgt', '*'),
        ('fun', 'test.ping'),
    )

    def _token(self):
        '''
        Return the token
        '''
        body = urlencode(self.auth_creds)
        request, response = self.request(
            '/login',
            method='POST',
            body=body,
            headers={
                'content-type': 'application/x-www-form-urlencoded'
            }
        )
        return response.headers['X-Auth-Token']

    def _add_job(self):
        '''
        Helper function to add a job to the job cache
        '''
        cmd = dict(self.low, **dict(self.auth_creds))
        body = urlencode(cmd)

        request, response = self.request('/run', method='POST', body=body,
            headers={
                'content-type': 'application/x-www-form-urlencoded'
        })
        self.assertEqual(response.status, '200 OK')

    def test_all_jobs(self):
        '''
        test query to /jobs returns job data
        '''
        self._add_job()

        request, response = self.request('/jobs', method='GET',
            headers={
                'Accept': 'application/json',
                'X-Auth-Token': self._token(),
        })

        resp = salt.utils.json.loads(salt.utils.stringutils.to_str(response.body[0]))
        self.assertIn('test.ping', str(resp['return']))
        self.assertEqual(response.status, '200 OK')



# -*- coding: utf-8 -*-
'''
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
'''

# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals
try:
    import pwd
    HAS_PWD = True
except ImportError:
    HAS_PWD = False

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import TestCase, skipIf
from tests.support.mock import (
    MagicMock,
    patch,
    NO_MOCK,
    NO_MOCK_REASON
)

# Import Salt Libs
import salt.modules.useradd as useradd
from salt.exceptions import CommandExecutionError


@skipIf(NO_MOCK, NO_MOCK_REASON)
class UserAddTestCase(TestCase, LoaderModuleMockMixin):
    '''
    Test cases for salt.modules.useradd
    '''
    def setup_loader_modules(self):
        return {useradd: {}}

    @classmethod
    def setUpClass(cls):
        cls.mock_pwall = {'gid': 0,
                          'groups': ['root'],
                          'home': '/root',
                          'name': 'root',
                          'passwd': 'x',
                          'shell': '/bin/bash',
                          'uid': 0,
                          'fullname': 'root',
                          'roomnumber': '',
                          'workphone': '',
                          'homephone': '',
                          'other': ''}

    @classmethod
    def tearDownClass(cls):
        del cls.mock_pwall

    # 'add' function tests: 1

    def test_add(self):
        '''
        Test for adding a user
        '''
        with patch.dict(useradd.__grains__, {'kernel': 'OpenBSD'}):
            mock_primary = MagicMock(return_value='Salt')
            with patch.dict(useradd.__salt__,
                            {'file.gid_to_group': mock_primary}):
                mock = MagicMock(return_value={'retcode': 0})
                with patch.dict(useradd.__salt__, {'cmd.run_all': mock}):
                    self.assertTrue(useradd.add('Salt'))

                mock = MagicMock(return_value={'retcode': 1})
                with patch.dict(useradd.__salt__, {'cmd.run_all': mock}):
                    self.assertFalse(useradd.add('Salt'))

    # 'getent' function tests: 2

    @skipIf(HAS_PWD is False, 'The pwd module is not available')
    def test_getent(self):
        '''
        Test if user.getent already have a value
        '''
        with patch('salt.modules.useradd.__context__', MagicMock(return_value='Salt')):
            self.assertTrue(useradd.getent())

    @skipIf(HAS_PWD is False, 'The pwd module is not available')
    def test_getent_user(self):
        '''
        Tests the return information on all users
        '''
        with patch('pwd.getpwall', MagicMock(return_value=[''])):
            ret = [{'gid': 0,
                    'groups': ['root'],
                    'home': '/root',
                    'name': 'root',
                    'passwd': 'x',
                    'shell': '/bin/bash',
                    'uid': 0,
                    'fullname': 'root',
                    'roomnumber': '',
                    'workphone': '',
                    'homephone': '',
                    'other': ''}]
            with patch('salt.modules.useradd._format_info', MagicMock(return_value=self.mock_pwall)):
                self.assertEqual(useradd.getent(), ret)

    # 'chuid' function tests: 1

    def test_chuid(self):
        '''
        Test if the uid of a user change
        '''
        mock = MagicMock(return_value={'uid': 11})
        with patch.object(useradd, 'info', mock):
            self.assertTrue(useradd.chuid('name', 11))

        mock_run = MagicMock(return_value=None)
        with patch.dict(useradd.__salt__, {'cmd.run': mock_run}):
            mock = MagicMock(side_effect=[{'uid': 11}, {'uid': 11}])
            with patch.object(useradd, 'info', mock):
                self.assertFalse(useradd.chuid('name', 22))

        with patch.dict(useradd.__salt__, {'cmd.run': mock_run}):
            mock = MagicMock(side_effect=[{'uid': 11}, {'uid': 22}])
            with patch.object(useradd, 'info', mock):
                self.assertTrue(useradd.chuid('name', 11))

    # 'chgid' function tests: 1

    def test_chgid(self):
        '''
        Test the default group of the user
        '''
        mock = MagicMock(return_value={'gid': 11})
        with patch.object(useradd, 'info', mock):
            self.assertTrue(useradd.chgid('name', 11))

        mock_run = MagicMock(return_value=None)
        with patch.dict(useradd.__salt__, {'cmd.run': mock_run}):
            mock = MagicMock(side_effect=[{'gid': 22}, {'gid': 22}])
            with patch.object(useradd, 'info', mock):
                self.assertFalse(useradd.chgid('name', 11))

        with patch.dict(useradd.__salt__, {'cmd.run': mock_run}):
            mock = MagicMock(side_effect=[{'gid': 11}, {'gid': 22}])
            with patch.object(useradd, 'info', mock):
                self.assertTrue(useradd.chgid('name', 11))

    # 'chshell' function tests: 1

    def test_chshell(self):
        '''
        Test the default shell of user
        '''
        mock = MagicMock(return_value={'shell': '/bin/bash'})
        with patch.object(useradd, 'info', mock):
            self.assertTrue(useradd.chshell('name', '/bin/bash'))

        mock_run = MagicMock(return_value=None)
        with patch.dict(useradd.__salt__, {'cmd.run': mock_run}):
            mock = MagicMock(side_effect=[{'shell': '/bin/bash'},
                                          {'shell': '/bin/bash'}])
            with patch.object(useradd, 'info', mock):
                self.assertFalse(useradd.chshell('name', '/usr/bash'))

        with patch.dict(useradd.__salt__, {'cmd.run': mock_run}):
            mock = MagicMock(side_effect=[{'shell': '/bin/bash'},
                                          {'shell': '/usr/bash'}])
            with patch.object(useradd, 'info', mock):
                self.assertTrue(useradd.chshell('name', '/bin/bash'))

    # 'chhome' function tests: 1

    def test_chhome(self):
        '''
        Test if home directory given is same as previous home directory
        '''
        mock = MagicMock(return_value={'home': '/root'})
        with patch.object(useradd, 'info', mock):
            self.assertTrue(useradd.chhome('name', '/root'))

        mock = MagicMock(return_value=None)
        with patch.dict(useradd.__salt__, {'cmd.run': mock}):
            mock = MagicMock(side_effect=[{'home': '/root'}, {'home': '/root'}])
            with patch.object(useradd, 'info', mock):
                self.assertFalse(useradd.chhome('name', '/user'))

        mock = MagicMock(return_value=None)
        with patch.dict(useradd.__salt__, {'cmd.run': mock}):
            mock = MagicMock(side_effect=[{'home': '/root'}, {'home': '/root'}])
            with patch.object(useradd, 'info', mock):
                self.assertTrue(useradd.chhome('name', '/root'))

    # 'chgroups' function tests: 1

    def test_chgroups(self):
        '''
        Test if user groups changed
        '''
        mock = MagicMock(return_value=['wheel', 'root'])
        with patch.object(useradd, 'list_groups', mock):
            self.assertTrue(useradd.chgroups('foo', 'wheel,root'))

        mock = MagicMock(return_value=['wheel', 'root'])
        with patch.object(useradd, 'list_groups', mock):
            with patch.dict(useradd.__grains__, {'kernel': 'OpenBSD'}):
                mock_runall = MagicMock(return_value={'retcode': False,
                                                      'stderr': ''})
                with patch.dict(useradd.__salt__, {'cmd.run_all': mock_runall}):
                    self.assertTrue(useradd.chgroups('foo', 'wheel,test,root'))

                mock_runall = MagicMock(return_value={'retcode': True,
                                                      'stderr': ''})
                with patch.dict(useradd.__salt__, {'cmd.run_all': mock_runall}):
                    self.assertFalse(useradd.chgroups('foo', 'wheel,test,root'))

    # 'chfullname' function tests: 1

    def test_chfullname(self):
        '''
        Test if the user's Full Name is changed
        '''
        mock = MagicMock(return_value=False)
        with patch.object(useradd, '_get_gecos', mock):
            self.assertFalse(useradd.chfullname('Salt', 'SaltStack'))

        mock = MagicMock(return_value={'fullname': 'SaltStack'})
        with patch.object(useradd, '_get_gecos', mock):
            self.assertTrue(useradd.chfullname('Salt', 'SaltStack'))

        mock = MagicMock(return_value={'fullname': 'SaltStack'})
        with patch.object(useradd, '_get_gecos', mock):
            mock = MagicMock(return_value=None)
            with patch.dict(useradd.__salt__, {'cmd.run': mock}):
                mock = MagicMock(return_value={'fullname': 'SaltStack2'})
                with patch.object(useradd, 'info', mock):
                    self.assertFalse(useradd.chfullname('Salt', 'SaltStack1'))

        mock = MagicMock(return_value={'fullname': 'SaltStack2'})
        with patch.object(useradd, '_get_gecos', mock):
            mock = MagicMock(return_value=None)
            with patch.dict(useradd.__salt__, {'cmd.run': mock}):
                mock = MagicMock(return_value={'fullname': 'SaltStack2'})
                with patch.object(useradd, 'info', mock):
                    self.assertFalse(useradd.chfullname('Salt', 'SaltStack1'))

    # 'chroomnumber' function tests: 1

    def test_chroomnumber(self):
        '''
        Test if the user's Room Number is changed
        '''
        mock = MagicMock(return_value=False)
        with patch.object(useradd, '_get_gecos', mock):
            self.assertFalse(useradd.chroomnumber('salt', 1))

        mock = MagicMock(return_value={'roomnumber': '1'})
        with patch.object(useradd, '_get_gecos', mock):
            self.assertTrue(useradd.chroomnumber('salt', 1))

        mock = MagicMock(return_value={'roomnumber': '2'})
        with patch.object(useradd, '_get_gecos', mock):
            mock = MagicMock(return_value=None)
            with patch.dict(useradd.__salt__, {'cmd.run': mock}):
                mock = MagicMock(return_value={'roomnumber': '3'})
                with patch.object(useradd, 'info', mock):
                    self.assertFalse(useradd.chroomnumber('salt', 1))

        mock = MagicMock(return_value={'roomnumber': '3'})
        with patch.object(useradd, '_get_gecos', mock):
            mock = MagicMock(return_value=None)
            with patch.dict(useradd.__salt__, {'cmd.run': mock}):
                mock = MagicMock(return_value={'roomnumber': '3'})
                with patch.object(useradd, 'info', mock):
                    self.assertFalse(useradd.chroomnumber('salt', 1))

    # 'chworkphone' function tests: 1

    def test_chworkphone(self):
        '''
        Test if the user's Work Phone is changed
        '''
        mock = MagicMock(return_value=False)
        with patch.object(useradd, '_get_gecos', mock):
            self.assertFalse(useradd.chworkphone('salt', 1))

        mock = MagicMock(return_value={'workphone': '1'})
        with patch.object(useradd, '_get_gecos', mock):
            self.assertTrue(useradd.chworkphone('salt', 1))

        mock = MagicMock(return_value={'workphone': '2'})
        with patch.object(useradd, '_get_gecos', mock):
            mock = MagicMock(return_value=None)
            with patch.dict(useradd.__salt__, {'cmd.run': mock}):
                mock = MagicMock(return_value={'workphone': '3'})
                with patch.object(useradd, 'info', mock):
                    self.assertFalse(useradd.chworkphone('salt', 1))

        mock = MagicMock(return_value={'workphone': '3'})
        with patch.object(useradd, '_get_gecos', mock):
            mock = MagicMock(return_value=None)
            with patch.dict(useradd.__salt__, {'cmd.run': mock}):
                mock = MagicMock(return_value={'workphone': '3'})
                with patch.object(useradd, 'info', mock):
                    self.assertFalse(useradd.chworkphone('salt', 1))

    # 'chhomephone' function tests: 1

    def test_chhomephone(self):
        '''
        Test if the user's Home Phone is changed
        '''
        mock = MagicMock(return_value=False)
        with patch.object(useradd, '_get_gecos', mock):
            self.assertFalse(useradd.chhomephone('salt', 1))

        mock = MagicMock(return_value={'homephone': '1'})
        with patch.object(useradd, '_get_gecos', mock):
            self.assertTrue(useradd.chhomephone('salt', 1))

        mock = MagicMock(return_value={'homephone': '2'})
        with patch.object(useradd, '_get_gecos', mock):
            mock = MagicMock(return_value=None)
            with patch.dict(useradd.__salt__, {'cmd.run': mock}):
                mock = MagicMock(return_value={'homephone': '3'})
                with patch.object(useradd, 'info', mock):
                    self.assertFalse(useradd.chhomephone('salt', 1))

        mock = MagicMock(return_value={'homephone': '3'})
        with patch.object(useradd, '_get_gecos', mock):
            mock = MagicMock(return_value=None)
            with patch.dict(useradd.__salt__, {'cmd.run': mock}):
                mock = MagicMock(return_value={'homephone': '3'})
                with patch.object(useradd, 'info', mock):
                    self.assertFalse(useradd.chhomephone('salt', 1))

    # 'chother' function tests: 1

    def test_chother(self):
        '''
        Test if the user's other GECOS attribute is changed
        '''
        mock = MagicMock(return_value=False)
        with patch.object(useradd, '_get_gecos', mock):
            self.assertFalse(useradd.chother('salt', 1))

        mock = MagicMock(return_value={'other': 'foobar'})
        with patch.object(useradd, '_get_gecos', mock):
            self.assertTrue(useradd.chother('salt', 'foobar'))

        mock = MagicMock(return_value={'other': 'foobar2'})
        with patch.object(useradd, '_get_gecos', mock):
            mock = MagicMock(return_value=None)
            with patch.dict(useradd.__salt__, {'cmd.run': mock}):
                mock = MagicMock(return_value={'other': 'foobar3'})
                with patch.object(useradd, 'info', mock):
                    self.assertFalse(useradd.chother('salt', 'foobar'))

        mock = MagicMock(return_value={'other': 'foobar3'})
        with patch.object(useradd, '_get_gecos', mock):
            mock = MagicMock(return_value=None)
            with patch.dict(useradd.__salt__, {'cmd.run': mock}):
                mock = MagicMock(return_value={'other': 'foobar3'})
                with patch.object(useradd, 'info', mock):
                    self.assertFalse(useradd.chother('salt', 'foobar'))

    # 'info' function tests: 1

    @skipIf(HAS_PWD is False, 'The pwd module is not available')
    def test_info(self):
        '''
        Test the user information
        '''
        self.assertEqual(useradd.info('username-that-doesnt-exist'), {})

        mock = MagicMock(return_value=pwd.struct_passwd(('_TEST_GROUP',
                                                         '*',
                                                         83,
                                                         83,
                                                         'AMaViS Daemon',
                                                         '/var/virusmails',
                                                         '/usr/bin/false')))
        with patch.object(pwd, 'getpwnam', mock):
            self.assertEqual(useradd.info('username-that-doesnt-exist')['name'], '_TEST_GROUP')

    # 'list_groups' function tests: 1

    def test_list_groups(self):
        '''
        Test if it return a list of groups the named user belongs to
        '''
        with patch('salt.utils.user.get_group_list', MagicMock(return_value='Salt')):
            self.assertEqual(useradd.list_groups('name'), 'Salt')

    # 'list_users' function tests: 1

    @skipIf(HAS_PWD is False, 'The pwd module is not available')
    def test_list_users(self):
        '''
        Test if it returns a list of all users
        '''
        self.assertTrue(useradd.list_users())

    # 'list_users' function tests: 1

    def test_rename(self):
        '''
        Test if the username for a named user changed
        '''
        mock = MagicMock(return_value=False)
        with patch.object(useradd, 'info', mock):
            self.assertRaises(CommandExecutionError, useradd.rename, 'salt', 1)

        mock = MagicMock(return_value=True)
        with patch.object(useradd, 'info', mock):
            self.assertRaises(CommandExecutionError, useradd.rename, 'salt', 1)

        mock = MagicMock(return_value=None)
        with patch.dict(useradd.__salt__, {'cmd.run': mock}):
            mock = MagicMock(side_effect=[{'name': ''}, False,
                                          {'name': 'salt'}])
            with patch.object(useradd, 'info', mock):
                self.assertTrue(useradd.rename('name', 'salt'))

        mock = MagicMock(return_value=None)
        with patch.dict(useradd.__salt__, {'cmd.run': mock}):
            mock = MagicMock(side_effect=[{'name': ''}, False, {'name': ''}])
            with patch.object(useradd, 'info', mock):
                self.assertFalse(useradd.rename('salt', 'salt'))

    def test_build_gecos_field(self):
        '''
        Test if gecos fields are built correctly (removing trailing commas)
        '''
        test_gecos = {'fullname': 'Testing',
                      'roomnumber': 1234,
                      'workphone': 22222,
                      'homephone': 99999}
        expected_gecos_fields = 'Testing,1234,22222,99999'
        self.assertEqual(useradd._build_gecos(test_gecos), expected_gecos_fields)
        test_gecos.pop('roomnumber')
        test_gecos.pop('workphone')
        expected_gecos_fields = 'Testing,,,99999'
        self.assertEqual(useradd._build_gecos(test_gecos), expected_gecos_fields)
        test_gecos.pop('homephone')
        expected_gecos_fields = 'Testing'
        self.assertEqual(useradd._build_gecos(test_gecos), expected_gecos_fields)



# -*- coding: utf-8 -*-
'''
Integration tests for the beacon states
'''

# Import Python Libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Testing Libs
from tests.support.case import ModuleCase
from tests.support.helpers import destructiveTest
from tests.support.mixins import SaltReturnAssertsMixin

import logging
log = logging.getLogger(__name__)


@destructiveTest
class BeaconStateTestCase(ModuleCase, SaltReturnAssertsMixin):
    '''
    Test beacon states
    '''
    def setUp(self):
        '''
        '''
        self.run_function('beacons.reset')

    def tearDown(self):
        self.run_function('beacons.reset')

    def test_present_absent(self):
        kwargs = {'/': '38%', 'interval': 5}
        ret = self.run_state(
            'beacon.present',
            name='diskusage',
            **kwargs
        )
        self.assertSaltTrueReturn(ret)

        ret = self.run_function('beacons.list', return_yaml=False)
        self.assertTrue('diskusage' in ret)
        self.assertTrue({'interval': 5} in ret['diskusage'])
        self.assertTrue({'/': '38%'} in ret['diskusage'])

        ret = self.run_state(
            'beacon.absent',
            name='diskusage',
        )
        self.assertSaltTrueReturn(ret)

        ret = self.run_function('beacons.list', return_yaml=False)
        self.assertEqual(ret, {'beacons': {}})



# -*- coding: utf-8 -*-

# Import python libs
from __future__ import absolute_import, print_function, unicode_literals
import os

# Import Salt Testing libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import TestCase, skipIf
from tests.support.mock import (
    MagicMock,
    patch
)
# Import salt libs
import salt.states.environ as envstate
import salt.modules.environ as envmodule
import salt.modules.reg
import salt.utils.platform


class TestEnvironState(TestCase, LoaderModuleMockMixin):

    def setup_loader_modules(self):
        loader_globals = {
            '__env__': 'base',
            '__opts__': {'test': False},
            '__salt__': {
                'environ.setenv': envmodule.setenv,
                'reg.read_value': salt.modules.reg.read_value,
            }
        }
        return {envstate: loader_globals, envmodule: loader_globals}

    def setUp(self):
        patcher = patch.dict(os.environ, {'INITIAL': 'initial'}, clear=True)
        patcher.start()

        def reset_environ(patcher):
            patcher.stop()

        self.addCleanup(reset_environ, patcher)

    def test_setenv(self):
        '''
        test that a subsequent calls of setenv changes nothing
        '''
        ret = envstate.setenv('test', 'value')
        self.assertEqual(ret['changes'], {'test': 'value'})

        ret = envstate.setenv('test', 'other')
        self.assertEqual(ret['changes'], {'test': 'other'})

        # once again with the same value
        ret = envstate.setenv('test', 'other')
        self.assertEqual(ret['changes'], {})

    @skipIf(not salt.utils.platform.is_windows(), 'Windows only')
    def test_setenv_permanent(self):
        '''
        test that we can set perminent environment variables (requires pywin32)
        '''
        with patch.dict(envmodule.__salt__, {'reg.set_value': MagicMock(), 'reg.delete_value': MagicMock()}):
            ret = envstate.setenv('test', 'value', permanent=True)
            self.assertEqual(ret['changes'], {'test': 'value'})
            envmodule.__salt__['reg.set_value'].assert_called_with("HKCU", "Environment", 'test', 'value')

            ret = envstate.setenv('test', False, false_unsets=True, permanent=True)
            self.assertEqual(ret['changes'], {'test': None})
            envmodule.__salt__['reg.delete_value'].assert_called_with("HKCU", "Environment", 'test')

    def test_setenv_dict(self):
        '''
        test that setenv can be invoked with dict
        '''
        ret = envstate.setenv('notimportant', {'test': 'value'})
        self.assertEqual(ret['changes'], {'test': 'value'})

    def test_setenv_int(self):
        '''
        test that setenv can not be invoked with int
        (actually it's anything other than strings and dict)
        '''
        ret = envstate.setenv('test', 1)
        self.assertEqual(ret['result'], False)

    def test_setenv_unset(self):
        '''
        test that ``false_unsets`` option removes variable from environment
        '''
        ret = envstate.setenv('test', 'value')
        self.assertEqual(ret['changes'], {'test': 'value'})

        ret = envstate.setenv('notimportant', {'test': False}, false_unsets=True)
        self.assertEqual(ret['changes'], {'test': None})
        self.assertEqual(envstate.os.environ, {'INITIAL': 'initial'})

    def test_setenv_clearall(self):
        '''
        test that ``clear_all`` option sets other values to ''
        '''
        ret = envstate.setenv('test', 'value', clear_all=True)
        self.assertEqual(ret['changes'], {'test': 'value', 'INITIAL': ''})
        if salt.utils.platform.is_windows():
            self.assertEqual(envstate.os.environ, {'TEST': 'value', 'INITIAL': ''})
        else:
            self.assertEqual(envstate.os.environ, {'test': 'value', 'INITIAL': ''})

    def test_setenv_clearall_with_unset(self):
        '''
        test that ``clear_all`` option combined with ``false_unsets``
        unsets other values from environment
        '''
        ret = envstate.setenv('test', 'value', false_unsets=True, clear_all=True)
        self.assertEqual(ret['changes'], {'test': 'value', 'INITIAL': None})
        if salt.utils.platform.is_windows():
            self.assertEqual(envstate.os.environ, {'TEST': 'value'})
        else:
            self.assertEqual(envstate.os.environ, {'test': 'value'})

    def test_setenv_unset_multi(self):
        '''
        test basically same things that above tests but with multiple values passed
        '''
        ret = envstate.setenv('notimportant', {'foo': 'bar'})
        self.assertEqual(ret['changes'], {'foo': 'bar'})

        with patch.dict(envstate.__salt__, {'reg.read_value': MagicMock()}):
            ret = envstate.setenv(
                'notimportant', {'test': False, 'foo': 'baz'}, false_unsets=True)
        self.assertEqual(ret['changes'], {'test': None, 'foo': 'baz'})
        if salt.utils.platform.is_windows():
            self.assertEqual(envstate.os.environ, {'INITIAL': 'initial', 'FOO': 'baz'})
        else:
            self.assertEqual(envstate.os.environ, {'INITIAL': 'initial', 'foo': 'baz'})

        with patch.dict(envstate.__salt__, {'reg.read_value': MagicMock()}):
            ret = envstate.setenv('notimportant', {'test': False, 'foo': 'bax'})
        self.assertEqual(ret['changes'], {'test': '', 'foo': 'bax'})
        if salt.utils.platform.is_windows():
            self.assertEqual(envstate.os.environ, {'INITIAL': 'initial', 'FOO': 'bax', 'TEST': ''})
        else:
            self.assertEqual(envstate.os.environ, {'INITIAL': 'initial', 'foo': 'bax', 'test': ''})

    def test_setenv_test_mode(self):
        '''
        test that imitating action returns good values
        '''
        with patch.dict(envstate.__opts__, {'test': True}):
            ret = envstate.setenv('test', 'value')
            self.assertEqual(ret['changes'], {'test': 'value'})
            ret = envstate.setenv('INITIAL', 'initial')
            self.assertEqual(ret['changes'], {})



# -*- coding: utf-8 -*-
'''
Microsoft Updates (KB) Management

This module provides the ability to enforce KB installations from files (.msu),
without WSUS or Windows Update

.. versionadded:: 2018.3.4
'''

# Import python libs
from __future__ import absolute_import, unicode_literals
import logging

# Import salt libs
import salt.utils.platform
import salt.utils.url
from salt.exceptions import SaltInvocationError

log = logging.getLogger(__name__)


# Define the module's virtual name
__virtualname__ = 'wusa'


def __virtual__():
    '''
    Load only on Windows
    '''
    if not salt.utils.platform.is_windows():
        return False, 'Only available on Windows systems'

    return __virtualname__


def installed(name, source):
    '''
    Ensure an update is installed on the minion

    Args:

        name(str):
            Name of the Windows KB ("KB123456")

        source (str):
            Source of .msu file corresponding to the KB

    Example:

    .. code-block:: yaml

        KB123456:
          wusa.installed:
            - source: salt://kb123456.msu
    '''
    ret = {'name': name,
           'changes': {},
           'result': False,
           'comment': ''}

    # Input validation
    if not name:
        raise SaltInvocationError('Must specify a KB "name"')
    if not source:
        raise SaltInvocationError('Must specify a "source" file to install')

    # Is the KB already installed
    if __salt__['wusa.is_installed'](name):
        ret['result'] = True
        ret['comment'] = '{0} already installed'.format(name)
        return ret

    # Check for test=True
    if __opts__['test'] is True:
        ret['result'] = None
        ret['comment'] = '{0} would be installed'.format(name)
        ret['result'] = None
        return ret

    # Cache the file
    cached_source_path = __salt__['cp.cache_file'](path=source, saltenv=__env__)
    if not cached_source_path:
        msg = 'Unable to cache {0} from saltenv "{1}"'.format(
            salt.utils.url.redact_http_basic_auth(source), __env__)
        ret['comment'] = msg
        return ret

    # Install the KB
    __salt__['wusa.install'](cached_source_path)

    # Verify successful install
    if __salt__['wusa.is_installed'](name):
        ret['comment'] = '{0} was installed'.format(name)
        ret['changes'] = {'old': False, 'new': True}
        ret['result'] = True
    else:
        ret['comment'] = '{0} failed to install'.format(name)

    return ret


def uninstalled(name):
    '''
    Ensure an update is uninstalled from the minion

    Args:

        name(str):
            Name of the Windows KB ("KB123456")

    Example:

    .. code-block:: yaml

        KB123456:
          wusa.uninstalled
    '''
    ret = {'name': name,
           'changes': {},
           'result': False,
           'comment': ''}

    # Is the KB already uninstalled
    if not __salt__['wusa.is_installed'](name):
        ret['result'] = True
        ret['comment'] = '{0} already uninstalled'.format(name)
        return ret

    # Check for test=True
    if __opts__['test'] is True:
        ret['result'] = None
        ret['comment'] = '{0} would be uninstalled'.format(name)
        ret['result'] = None
        return ret

    # Uninstall the KB
    __salt__['wusa.uninstall'](name)

    # Verify successful uninstall
    if not __salt__['wusa.is_installed'](name):
        ret['comment'] = '{0} was uninstalled'.format(name)
        ret['changes'] = {'old': True, 'new': False}
        ret['result'] = True
    else:
        ret['comment'] = '{0} failed to uninstall'.format(name)

    return ret



# -*- coding: utf-8 -*-
r'''
A salt util for modifying firewall settings.

.. versionadded:: 2018.3.4
.. versionadded:: 2019.2.0

This util allows you to modify firewall settings in the local group policy in
addition to the normal firewall settings. Parameters are taken from the
netsh advfirewall prompt.

.. note::
    More information can be found in the advfirewall context in netsh. This can
    be access by opening a netsh prompt. At a command prompt type the following:

    c:\>netsh
    netsh>advfirewall
    netsh advfirewall>set help
    netsh advfirewall>set domain help

Usage:

.. code-block:: python

    import salt.utils.win_lgpo_netsh

    # Get the inbound/outbound firewall settings for connections on the
    # local domain profile
    salt.utils.win_lgpo_netsh.get_settings(profile='domain',
                                           section='firewallpolicy')

    # Get the inbound/outbound firewall settings for connections on the
    # domain profile as defined by local group policy
    salt.utils.win_lgpo_netsh.get_settings(profile='domain',
                                           section='firewallpolicy',
                                           store='lgpo')

    # Get all firewall settings for connections on the domain profile
    salt.utils.win_lgpo_netsh.get_all_settings(profile='domain')

    # Get all firewall settings for connections on the domain profile as
    # defined by local group policy
    salt.utils.win_lgpo_netsh.get_all_settings(profile='domain', store='lgpo')

    # Get all firewall settings for all profiles
    salt.utils.win_lgpo_netsh.get_all_settings()

    # Get all firewall settings for all profiles as defined by local group
    # policy
    salt.utils.win_lgpo_netsh.get_all_settings(store='lgpo')

    # Set the inbound setting for the domain profile to block inbound
    # connections
    salt.utils.win_lgpo_netsh.set_firewall_settings(profile='domain',
                                                    inbound='blockinbound')

    # Set the outbound setting for the domain profile to allow outbound
    # connections
    salt.utils.win_lgpo_netsh.set_firewall_settings(profile='domain',
                                                    outbound='allowoutbound')

    # Set inbound/outbound settings for the domain profile in the group
    # policy to block inbound and allow outbound
    salt.utils.win_lgpo_netsh.set_firewall_settings(profile='domain',
                                                    inbound='blockinbound',
                                                    outbound='allowoutbound',
                                                    store='lgpo')
'''
# Import Python libs
from __future__ import absolute_import, unicode_literals, print_function
from textwrap import dedent
import logging
import os
import re
import socket
import tempfile

import salt.modules.cmdmod
from salt.exceptions import CommandExecutionError
from salt.ext.six.moves import zip

log = logging.getLogger(__name__)
__hostname__ = socket.gethostname()
__virtualname__ = 'netsh'


# Although utils are often directly imported, it is also possible to use the
# loader.
def __virtual__():
    '''
    Only load if on a Windows system
    '''
    if not salt.utils.platform.is_windows():
        return False, 'This utility only available on Windows'

    return __virtualname__


def _netsh_file(content):
    '''
    helper function to get the results of ``netsh -f content.txt``

    Running ``netsh`` will drop you into a ``netsh`` prompt where you can issue
    ``netsh`` commands. You can put a series of commands in an external file and
    run them as if from a ``netsh`` prompt using the ``-f`` switch. That's what
    this function does.

    Args:

        content (str):
            The contents of the file that will be run by the ``netsh -f``
            command

    Returns:
        str: The text returned by the netsh command
    '''
    with tempfile.NamedTemporaryFile(mode='w',
                                     prefix='salt-',
                                     suffix='.netsh',
                                     delete=False) as fp:
        fp.write(content)
    try:
        log.debug('{0}:\n{1}'.format(fp.name, content))
        return salt.modules.cmdmod.run('netsh -f {0}'.format(fp.name), python_shell=True)
    finally:
        os.remove(fp.name)


def _netsh_command(command, store):
    if store.lower() not in ('local', 'lgpo'):
        raise ValueError('Incorrect store: {0}'.format(store))
    # set the store for local or lgpo
    if store.lower() == 'local':
        netsh_script = dedent('''\
            advfirewall
            set store local 
            {0}
        '''.format(command))
    else:
        netsh_script = dedent('''\
            advfirewall
            set store gpo = {0}
            {1}
        '''.format(__hostname__, command))
    return _netsh_file(content=netsh_script).splitlines()


def get_settings(profile, section, store='local'):
    '''
    Get the firewall property from the specified profile in the specified store
    as returned by ``netsh advfirewall``.

    Args:

        profile (str):
            The firewall profile to query. Valid options are:

            - domain
            - public
            - private

        section (str):
            The property to query within the selected profile. Valid options
            are:

            - firewallpolicy : inbound/outbound behavior
            - logging : firewall logging settings
            - settings : firewall properties
            - state : firewalls state (on | off)

        store (str):
            The store to use. This is either the local firewall policy or the
            policy defined by local group policy. Valid options are:

            - lgpo
            - local

            Default is ``local``

    Returns:
        dict: A dictionary containing the properties for the specified profile

    Raises:
        CommandExecutionError: If an error occurs
        ValueError: If the parameters are incorrect
    '''
    # validate input
    if profile.lower() not in ('domain', 'public', 'private'):
        raise ValueError('Incorrect profile: {0}'.format(profile))
    if section.lower() not in ('state', 'firewallpolicy', 'settings', 'logging'):
        raise ValueError('Incorrect section: {0}'.format(section))
    if store.lower() not in ('local', 'lgpo'):
        raise ValueError('Incorrect store: {0}'.format(store))
    command = 'show {0}profile {1}'.format(profile, section)
    # run it
    results = _netsh_command(command=command, store=store)
    # sample output:
    # Domain Profile Settings:
    # ----------------------------------------------------------------------
    # LocalFirewallRules                    N/A (GPO-store only)
    # LocalConSecRules                      N/A (GPO-store only)
    # InboundUserNotification               Disable
    # RemoteManagement                      Disable
    # UnicastResponseToMulticast            Enable

    # if it's less than 3 lines it failed
    if len(results) < 3:
        raise CommandExecutionError('Invalid results: {0}'.format(results))
    ret = {}
    # Skip the first 2 lines. Add everything else to a dictionary
    for line in results[3:]:
        ret.update(dict(list(zip(*[iter(re.split(r"\s{2,}", line))]*2))))

    # Remove spaces from the values so that `Not Configured` is detected
    # correctly
    for item in ret:
        ret[item] = ret[item].replace(' ', '')

    # special handling for firewallpolicy
    if section == 'firewallpolicy':
        inbound, outbound = ret['Firewall Policy'].split(',')
        return {'Inbound': inbound, 'Outbound': outbound}

    return ret


def get_all_settings(profile, store='local'):
    '''
    Gets all the properties for the specified profile in the specified store

    Args:

        profile (str):
            The firewall profile to query. Valid options are:

            - domain
            - public
            - private

        store (str):
            The store to use. This is either the local firewall policy or the
            policy defined by local group policy. Valid options are:

            - lgpo
            - local

            Default is ``local``

    Returns:
        dict: A dictionary containing the specified settings
    '''
    ret = dict()
    ret.update(get_settings(profile=profile, section='state', store=store))
    ret.update(get_settings(profile=profile, section='firewallpolicy', store=store))
    ret.update(get_settings(profile=profile, section='settings', store=store))
    ret.update(get_settings(profile=profile, section='logging', store=store))
    return ret


def get_all_profiles(store='local'):
    '''
    Gets all properties for all profiles in the specified store

    Args:

        store (str):
            The store to use. This is either the local firewall policy or the
            policy defined by local group policy. Valid options are:

            - lgpo
            - local

            Default is ``local``

    Returns:
        dict: A dictionary containing the specified settings for each profile
    '''
    return {
        'Domain Profile': get_all_settings(profile='domain', store=store),
        'Private Profile': get_all_settings(profile='private', store=store),
        'Public Profile': get_all_settings(profile='public', store=store)
    }


def set_firewall_settings(profile,
                          inbound=None,
                          outbound=None,
                          store='local'):
    '''
    Set the firewall inbound/outbound settings for the specified profile and
    store

    Args:

        profile (str):
            The firewall profile to configure. Valid options are:

            - domain
            - public
            - private

        inbound (str):
            The inbound setting. If ``None`` is passed, the setting will remain
            unchanged. Valid values are:

            - blockinbound
            - blockinboundalways
            - allowinbound
            - notconfigured

            Default is ``None``

        outbound (str):
            The outbound setting. If ``None`` is passed, the setting will remain
            unchanged. Valid values are:

            - allowoutbound
            - blockoutbound
            - notconfigured

            Default is ``None``

        store (str):
            The store to use. This is either the local firewall policy or the
            policy defined by local group policy. Valid options are:

            - lgpo
            - local

            Default is ``local``

    Returns:
        bool: ``True`` if successful

    Raises:
        CommandExecutionError: If an error occurs
        ValueError: If the parameters are incorrect
    '''
    # Input validation
    if profile.lower() not in ('domain', 'public', 'private'):
        raise ValueError('Incorrect profile: {0}'.format(profile))
    if inbound and inbound.lower() not in ('blockinbound',
                                           'blockinboundalways',
                                           'allowinbound',
                                           'notconfigured'):
        raise ValueError('Incorrect inbound value: {0}'.format(inbound))
    if outbound and outbound.lower() not in ('allowoutbound',
                                             'blockoutbound',
                                             'notconfigured'):
        raise ValueError('Incorrect outbound value: {0}'.format(outbound))
    if not inbound and not outbound:
        raise ValueError('Must set inbound or outbound')

    # You have to specify inbound and outbound setting at the same time
    # If you're only specifying one, you have to get the current setting for the
    # other
    if not inbound or not outbound:
        ret = get_settings(profile=profile,
                           section='firewallpolicy',
                           store=store)
        if not inbound:
            inbound = ret['Inbound']
        if not outbound:
            outbound = ret['Outbound']

    command = 'set {0}profile firewallpolicy {1},{2}' \
              ''.format(profile, inbound, outbound)

    results = _netsh_command(command=command, store=store)

    if results:
        raise CommandExecutionError('An error occurred: {0}'.format(results))

    return True


def set_logging_settings(profile, setting, value, store='local'):
    '''
    Configure logging settings for the Windows firewall.

    Args:

        profile (str):
            The firewall profile to configure. Valid options are:

            - domain
            - public
            - private

        setting (str):
            The logging setting to configure. Valid options are:

            - allowedconnections
            - droppedconnections
            - filename
            - maxfilesize

        value (str):
            The value to apply to the setting. Valid values are dependent upon
            the setting being configured. Valid options are:

            allowedconnections:

                - enable
                - disable
                - notconfigured

            droppedconnections:

                - enable
                - disable
                - notconfigured

            filename:

                - Full path and name of the firewall log file
                - notconfigured

            maxfilesize:

                - 1 - 32767 (Kb)
                - notconfigured

        store (str):
            The store to use. This is either the local firewall policy or the
            policy defined by local group policy. Valid options are:

            - lgpo
            - local

            Default is ``local``

    Returns:
        bool: ``True`` if successful

    Raises:
        CommandExecutionError: If an error occurs
        ValueError: If the parameters are incorrect
    '''
    # Input validation
    if profile.lower() not in ('domain', 'public', 'private'):
        raise ValueError('Incorrect profile: {0}'.format(profile))
    if setting.lower() not in ('allowedconnections',
                               'droppedconnections',
                               'filename',
                               'maxfilesize'):
        raise ValueError('Incorrect setting: {0}'.format(setting))
    if setting.lower() in ('allowedconnections', 'droppedconnections'):
        if value.lower() not in ('enable', 'disable', 'notconfigured'):
            raise ValueError('Incorrect value: {0}'.format(value))
    # TODO: Consider adding something like the following to validate filename
    # https://stackoverflow.com/questions/9532499/check-whether-a-path-is-valid-in-python-without-creating-a-file-at-the-paths-ta
    if setting.lower() == 'maxfilesize':
        if value.lower() != 'notconfigured':
            # Must be a number between 1 and 32767
            try:
                int(value)
            except ValueError:
                raise ValueError('Incorrect value: {0}'.format(value))
            if not 1 <= int(value) <= 32767:
                raise ValueError('Incorrect value: {0}'.format(value))
    # Run the command
    command = 'set {0}profile logging {1} {2}'.format(profile, setting, value)
    results = _netsh_command(command=command, store=store)

    # A successful run should return an empty list
    if results:
        raise CommandExecutionError('An error occurred: {0}'.format(results))

    return True


def set_settings(profile, setting, value, store='local'):
    '''
    Configure firewall settings.

    Args:

        profile (str):
            The firewall profile to configure. Valid options are:

            - domain
            - public
            - private

        setting (str):
            The firewall setting to configure. Valid options are:

            - localfirewallrules
            - localconsecrules
            - inboundusernotification
            - remotemanagement
            - unicastresponsetomulticast

        value (str):
            The value to apply to the setting. Valid options are

            - enable
            - disable
            - notconfigured

        store (str):
            The store to use. This is either the local firewall policy or the
            policy defined by local group policy. Valid options are:

            - lgpo
            - local

            Default is ``local``

    Returns:
        bool: ``True`` if successful

    Raises:
        CommandExecutionError: If an error occurs
        ValueError: If the parameters are incorrect
    '''
    # Input validation
    if profile.lower() not in ('domain', 'public', 'private'):
        raise ValueError('Incorrect profile: {0}'.format(profile))
    if setting.lower() not in ('localfirewallrules',
                               'localconsecrules',
                               'inboundusernotification',
                               'remotemanagement',
                               'unicastresponsetomulticast'):
        raise ValueError('Incorrect setting: {0}'.format(setting))
    if value.lower() not in ('enable', 'disable', 'notconfigured'):
        raise ValueError('Incorrect value: {0}'.format(value))

    # Run the command
    command = 'set {0}profile settings {1} {2}'.format(profile, setting, value)
    results = _netsh_command(command=command, store=store)

    # A successful run should return an empty list
    if results:
        raise CommandExecutionError('An error occurred: {0}'.format(results))

    return True


def set_state(profile, state, store='local'):
    '''
    Configure the firewall state.

    Args:

        profile (str):
            The firewall profile to configure. Valid options are:

            - domain
            - public
            - private

        state (str):
            The firewall state. Valid options are:

            - on
            - off
            - notconfigured

        store (str):
            The store to use. This is either the local firewall policy or the
            policy defined by local group policy. Valid options are:

            - lgpo
            - local

            Default is ``local``

    Returns:
        bool: ``True`` if successful

    Raises:
        CommandExecutionError: If an error occurs
        ValueError: If the parameters are incorrect
    '''
    # Input validation
    if profile.lower() not in ('domain', 'public', 'private'):
        raise ValueError('Incorrect profile: {0}'.format(profile))
    if state.lower() not in ('on', 'off', 'notconfigured'):
        raise ValueError('Incorrect state: {0}'.format(state))

    # Run the command
    command = 'set {0}profile state {1}'.format(profile, state)
    results = _netsh_command(command=command, store=store)

    # A successful run should return an empty list
    if results:
        raise CommandExecutionError('An error occurred: {0}'.format(results))

    return True



# coding: utf-8

# Python libs
from __future__ import absolute_import
from collections import namedtuple

# Salt testing libs
from tests.support.unit import skipIf, TestCase
from tests.support.mock import NO_MOCK, NO_MOCK_REASON, patch, MagicMock
from tests.support.mixins import LoaderModuleMockMixin

# Salt libs
import salt.beacons.network_info as network_info

import logging
log = logging.getLogger(__name__)

STUB_NET_IO_COUNTERS = {'eth0': namedtuple('snetio',
                                           'bytes_sent bytes_recv \
                                           packets_sent packets_recv \
                                           errin errout \
                                           dropin \
                                           dropout')(93662618, 914626664,
                                                     465694, 903802, 0,
                                                     0, 0, 0)}


@skipIf(NO_MOCK, NO_MOCK_REASON)
class NetworkInfoBeaconTestCase(TestCase, LoaderModuleMockMixin):
    '''
    Test case for salt.beacons.network_info
    '''

    def setup_loader_modules(self):
        return {
            network_info: {
                '__context__': {},
                '__salt__': {},
            }
        }

    def test_non_list_config(self):
        config = {}

        ret = network_info.validate(config)

        self.assertEqual(ret, (False, 'Configuration for network_info beacon'
                                      ' must be a list.'))

    def test_empty_config(self):
        config = [{}]

        ret = network_info.validate(config)

        self.assertEqual(ret, (True, 'Valid beacon configuration'))

    def test_network_info_equal(self):
        with patch('salt.utils.psutil_compat.net_io_counters',
                   MagicMock(return_value=STUB_NET_IO_COUNTERS)):
            config = [{'interfaces': {'eth0': {'type': 'equal',
                                               'bytes_sent': 914626664,
                                               'bytes_recv': 93662618,
                                               'packets_sent': 465694,
                                               'packets_recv': 903802,
                                               'errin': 0,
                                               'errout': 0,
                                               'dropin': 0,
                                               'dropout': 0}}}]

            ret = network_info.validate(config)

            self.assertEqual(ret, (True, 'Valid beacon configuration'))

            _expected_return = [{'interface': 'eth0',
                                 'network_info': {'bytes_recv': 914626664,
                                                  'bytes_sent': 93662618,
                                                  'dropin': 0,
                                                  'dropout': 0,
                                                  'errin': 0,
                                                  'errout': 0,
                                                  'packets_recv': 903802,
                                                  'packets_sent': 465694}}]

            ret = network_info.beacon(config)
            self.assertEqual(ret, _expected_return)

    def test_network_info_greater_than(self):
        with patch('salt.utils.psutil_compat.net_io_counters',
                   MagicMock(return_value=STUB_NET_IO_COUNTERS)):
            config = [{'interfaces': {'eth0': {'type': 'greater',
                                               'bytes_sent': 100000,
                                               'bytes_recv': 100000,
                                               'packets_sent': 100000,
                                               'packets_recv': 100000,
                                               'errin': 0,
                                               'errout': 0,
                                               'dropin': 0,
                                               'dropout': 0}}}]

            ret = network_info.validate(config)

            self.assertEqual(ret, (True, 'Valid beacon configuration'))

            _expected_return = [{'interface': 'eth0',
                                 'network_info': {'bytes_recv': 914626664,
                                                  'bytes_sent': 93662618,
                                                  'dropin': 0,
                                                  'dropout': 0,
                                                  'errin': 0,
                                                  'errout': 0,
                                                  'packets_recv': 903802,
                                                  'packets_sent': 465694}}]

            ret = network_info.beacon(config)
            self.assertEqual(ret, _expected_return)



# -*- coding: utf-8 -*-

# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Testing libs
from tests.support.unit import TestCase, skipIf
from tests.support.mixins import AdaptedConfigurationTestCaseMixin

# Import Salt libs
import salt.auth
import salt.wheel
import salt.utils.platform


class WheelModuleTest(TestCase, AdaptedConfigurationTestCaseMixin):

    eauth_creds = {
        'username': 'saltdev_auto',
        'password': 'saltdev',
        'eauth': 'auto',
    }

    def setUp(self):
        '''
        Configure an eauth user to test with
        '''
        self.wheel = salt.wheel.Wheel(dict(self.get_config('client_config')))

    def tearDown(self):
        del self.wheel

    def test_master_call(self):
        '''
        Test executing master_call with lowdata

        The choice of using key.list_all for this is arbitrary and should be
        changed to some mocked function that is more testing friendly.
        '''
        low = {
            'client': 'wheel',
            'fun': 'key.list_all',
            'print_event': False
        }
        low.update(self.eauth_creds)

        self.wheel.master_call(**low)

    def test_token(self):
        '''
        Test executing master_call with lowdata

        The choice of using key.list_all for this is arbitrary and should be
        changed to some mocked function that is more testing friendly.
        '''
        auth = salt.auth.LoadAuth(dict(self.get_config('client_config')))
        token = auth.mk_token(self.eauth_creds)

        token = auth.mk_token({
            'username': 'saltdev_auto',
            'password': 'saltdev',
            'eauth': 'auto',
        })

        self.wheel.master_call(**{
            'client': 'wheel',
            'fun': 'key.list_all',
            'token': token['token'],
            'print_event': False,
        })

    def test_cmd_sync(self):
        low = {
            'client': 'wheel',
            'fun': 'key.list_all',
            'print_event': False,
        }
        low.update(self.eauth_creds)

        self.wheel.cmd_sync(low)

    # Remove this skipIf when Issue #39616 is resolved
    # https://github.com/saltstack/salt/issues/39616
    @skipIf(salt.utils.platform.is_windows(),
            'Causes pickling error on Windows: Issue #39616')
    def test_cmd_async(self):
        low = {
            'client': 'wheel_async',
            'fun': 'key.list_all',
            'print_event': False,
        }
        low.update(self.eauth_creds)

        self.wheel.cmd_async(low)

    def test_cmd_sync_w_arg(self):
        low = {
            'fun': 'key.finger',
            'match': '*',
            'print_event': False,
        }
        low.update(self.eauth_creds)

        ret = self.wheel.cmd_sync(low)
        self.assertIn('return', ret.get('data', {}))

    def test_wildcard_auth(self):
        low = {
            'username': 'the_s0und_of_t3ch',
            'password': 'willrockyou',
            'eauth': 'auto',
            'fun': 'key.list_all',
            'print_event': False,
        }

        self.wheel.cmd_sync(low)



# -*- coding: utf-8 -*-
'''
    tests.unit.cloud.test_libcloudfuncs
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'''

# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Testing Libs
from tests.support.unit import TestCase

# Import Salt Libs
import salt.cloud.libcloudfuncs as libcloud


class LibcloudTestCase(TestCase):
    def test_node_state_libcloud_020(self):
        state = libcloud.node_state(2)
        self.assertEqual('TERMINATED', state)

    def test_node_state_libcloud_100(self):
        state = libcloud.node_state('terminated')
        self.assertEqual('TERMINATED', state)



# -*- coding: utf-8 -*-
'''
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
'''
# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import skipIf, TestCase
from tests.support.mock import NO_MOCK, NO_MOCK_REASON, MagicMock, patch

# Import Salt Libs
import salt.states.boto_asg as boto_asg


@skipIf(NO_MOCK, NO_MOCK_REASON)
class BotoAsgTestCase(TestCase, LoaderModuleMockMixin):
    '''
    Test cases for salt.states.boto_asg
    '''
    # 'present' function tests: 1

    def setup_loader_modules(self):
        return {boto_asg: {}}

    maxSize = None

    def test_present(self):
        '''
        Test to ensure the autoscale group exists.
        '''
        name = 'myasg'
        launch_config_name = 'mylc'
        availability_zones = ['us-east-1a', 'us-east-1b']
        min_size = 1
        max_size = 1

        ret = {'name': name,
               'result': None,
               'changes': {},
               'comment': ''}

        mock = MagicMock(side_effect=[False, {'min_size': 2}, ['']])
        with patch.dict(boto_asg.__salt__, {'boto_asg.get_config': mock}):
            with patch.dict(boto_asg.__opts__, {'test': True}):
                comt = 'Autoscale group set to be created.'
                ret.update({'comment': comt})
                with patch.dict(boto_asg.__salt__,
                                {'config.option': MagicMock(return_value={})}):
                    self.assertDictEqual(
                        boto_asg.present(
                            name,
                            launch_config_name,
                            availability_zones,
                            min_size,
                            max_size
                        ),
                        ret
                    )

                def magic_side_effect(value):
                    if isinstance(value, int):
                        if value == 1:
                            return 4
                        return value
                    return ''

                comt = 'Autoscale group set to be updated.'
                ret.update({'comment': comt, 'result': None})
                ret.update({'changes': {'new': {'min_size': 4},
                                        'old': {'min_size': 2}}})
                utils_ordered_mock = MagicMock(
                    side_effect=magic_side_effect
                )
                with patch.dict(boto_asg.__salt__,
                                {'config.option': MagicMock(return_value={})}):
                    with patch.dict(boto_asg.__utils__,
                                    {'boto3.ordered': utils_ordered_mock}):
                        call_ret = boto_asg.present(
                            name,
                            launch_config_name,
                            availability_zones,
                            min_size,
                            max_size
                        )
                        self.assertDictEqual(call_ret, ret)

                with patch.dict(boto_asg.__salt__,
                                {'config.option': MagicMock(return_value={})}):
                    with patch.dict(boto_asg.__utils__,
                                    {'boto3.ordered': MagicMock(return_value='')}):
                        comt = 'Autoscale group present. '
                        ret.update({'comment': comt, 'result': True})
                        ret.update({'changes': {}})
                        self.assertDictEqual(
                            boto_asg.present(
                                name,
                                launch_config_name,
                                availability_zones,
                                min_size,
                                max_size
                            ),
                            ret
                        )

    # 'absent' function tests: 1

    def test_absent(self):
        '''
        Test to ensure the named autoscale group is deleted.
        '''
        name = 'myasg'

        ret = {'name': name,
               'result': None,
               'changes': {},
               'comment': ''}

        mock = MagicMock(side_effect=[True, False])
        with patch.dict(boto_asg.__salt__, {'boto_asg.get_config': mock}):
            with patch.dict(boto_asg.__opts__, {'test': True}):
                comt = ('Autoscale group set to be deleted.')
                ret.update({'comment': comt})
                self.assertDictEqual(boto_asg.absent(name), ret)

                comt = ('Autoscale group does not exist.')
                ret.update({'comment': comt, 'result': True})
                self.assertDictEqual(boto_asg.absent(name), ret)
